#include "ExampleTestCase.h"

CPPUNIT_TEST_SUITE_REGISTRATION( ExampleTestCase );


void ExampleTestCase::setUp()
{
	//create and initialize client
	client = new BustaNetClient();
	client->init();
	// creation of a client initializes SDL (no need to reinitialize it afterwards)

	//create "server" socket
	SDLNet_ResolveHost(&serverIP, NULL, 65535);
	servsock = SDLNet_TCP_Open(&serverIP);

	//connect to "server"
	CPPUNIT_ASSERT_MESSAGE("Connecting to server", client->connect("127.0.0.1", 65535) == 0);

	//accept the connection from the client
	SDLNet_SocketReady(servsock);
	clientsock = SDLNet_TCP_Accept(servsock);
	CPPUNIT_ASSERT_MESSAGE("Server received client connection", clientsock != 0);
}

void ExampleTestCase::tearDown()
{
	client->disconnect();			// client disconnects
	
	char type;								// msg type to receive
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT_MESSAGE("Client Disconnect MSG received by Server",
		SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );					// assert msg received
	CPPUNIT_ASSERT_MESSAGE("Disconnect msg received", type == 'd' );	// assert disconnect msg type

	SDLNet_TCP_Close(servsock);
	delete client;
}

void ExampleTestCase::testsendCharSelect()
{
	// ------------------------------ Create Character selection to be sent -----------------------------
	BustaModelCharacter aChar = C5;		// character selected is C5

	// ------------------------------ Send selection using client ---------------------------------------
	client->sendCharSelect(aChar);

	// ------------------------------ "Server" receives the selection -----------------------------------
	// assert that selection is received as intended
	char type;				// the type of the message received
	cs_character_select cs;	// the message to be received

	// receive the type of the message
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );	// receive the type
	CPPUNIT_ASSERT( type == 'b' );			// assert that the message is of the right type

	// receive the message
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &cs, sizeof(cs_character_select)) == sizeof(cs_character_select) );
	CPPUNIT_ASSERT( cs.character == C5 );	// assert that correct selection received
}

void ExampleTestCase::testsendCursor()
{
	// ------------------------------ Create cursor to be sent -----------------------------
	BustaModelPosition* position = new BustaModelPosition(2,6);	// cursor at (2,6)

	// ------------------------------ Send cursor using client ---------------------------------------
	client->sendCursor(position);

	// ------------------------------ "Server" receives the cursor -----------------------------------
	// assert that selection is received as intended
	char type;					// type of the message received
	cs_character_cursor cur;	// the cursor received in the message

	// receive the type of the message
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) == 1 );	// receive the type
	CPPUNIT_ASSERT( type == 'c' );									// assert that it is right type

	// receive the message
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &cur, sizeof(cs_character_cursor)) == sizeof(cs_character_cursor) );
	CPPUNIT_ASSERT( cur.cursorLoc.x1 == 2 );		// assert that cursor is at right position
	CPPUNIT_ASSERT( cur.cursorLoc.y1 == 6 );
}

void ExampleTestCase::testsendGameOver()
{
	// ------------------------------ Send game over using client ---------------------------------------
	client->sendGameOver();

	// ------------------------------ "Server" receives the game over msg -----------------------------------
	// assert that message is correctly received
	char type;		// the type of the message

	// receive the type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );	// assert that we received the type
	CPPUNIT_ASSERT( type == 'e' );
}

void ExampleTestCase::testsendOpponentViewReq()
{
	// ------------------------------ Send selection using client ---------------------------------------
	client->sendOpponentViewReq(5);		// request view of player 5's gamestate

	// ------------------------------ "Server" receives the selection -----------------------------------
	// assert that selection is received as intended
	char type;						// type of the message
	cs_opponent_view_select ovs;	// the message to be received

	// receive the type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );	// assert msg received
	CPPUNIT_ASSERT( type == 'f' );									// assert right type

	// receive the message
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &ovs, sizeof(cs_opponent_view_select)) > 0 );	// assert msg received
	CPPUNIT_ASSERT( ovs.opponentID == 5 );			// assert correct request id number
}

void ExampleTestCase::testsendReady()
{
	// ------------------------------ Send ready using client ---------------------------------------
	client->sendReady();

	// ------------------------------ "Server" receives the ready msg -----------------------------------
	// assert that the message is of the right type
	char type;			// type of the message

	// receive the message type
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );	// assert msg received
	CPPUNIT_ASSERT( type == 'g' );									// assert correct msg type
}

void ExampleTestCase::testsendSandwich()
{
	// ------------------------------ Create sandwich to be sent -----------------------------
	SpecialSandwich sand = CONFUSE;

	// ------------------------------ Send sandwich using client ---------------------------------------
	client->sendSandwich(3, sand);		// send sandwich to opponent 3

	// ------------------------------ "Server" receives the sandwich -----------------------------------
	// assert that sandwich is received as intended
	char type;				// type of message
	cs_sandwich_send cs;	// sandwich message

	// receive the type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );		// assert type received
	CPPUNIT_ASSERT( type == 'h' );										// assert correct msg type

	// receive the sandwich msg
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &cs, sizeof(cs_sandwich_send)) > 0 );	// assert msg received
	CPPUNIT_ASSERT( cs.targetPlayer == 3 );		// correct opponent
	CPPUNIT_ASSERT( cs.sandwich == CONFUSE );	// correct sandwich sent
}

void ExampleTestCase::testsendState()
{
	// ------------------------------ Create Gamestate to be sent -----------------------------------
	BustaModelGameState* gs = new BustaModelGameState();	// create instance of gamestate to send
	
	/* Initialize board of ingredients in game state */
	for (int i = 0; i < HEIGHT; i++)
	{
		for (int j = 0; j < WIDTH; j++)
		{
			// entire board is of type TOMATO with state FRESH
			(gs->getBoard(i,j))->setState(FRESH);
			(gs->getBoard(i,j))->setType(TOMATO);
		}
	}

	// chef has health 100
	(gs->getChef())->setHealth(100);

	// tip jar has size of 50 and is currently at 25
	(gs->getTipJar())->setSize(50);
	(gs->getTipJar())->addTips(25);

	// cursor alignment 0, modifier 0, poition (5,5)
	(gs->getCursor())->setAlignment(0);
	(gs->getCursor())->setModifier(0);
	(gs->getCursor())->setPosition(5, 5);

	// customer orders (all arrays of 5 onions with state 5)
	// customer time all set at 10
	for (int cust = 0; cust < NUMCUSTOMERS; cust++)
	{
		for (int orderIndex = 0; orderIndex < WIDTH; orderIndex++)
		{
			((gs->getCustomer(cust))->getOrder())[orderIndex]->setType(ONION);
			((gs->getCustomer(cust))->getOrder())[orderIndex]->setState(FRESH);
		}

		//(gs->getCustomer(cust))->setTime(10);
	}

	// ------------------------------ Send gamestate using client -----------------------------------

	client->sendState(gs);

	// ------------------------------ "Server" receives the gamestate -----------------------------------
	// assert that all set gamestate variables are as they should be
	char type;			// type of the message received
	netGameState ngs;	// data to be received storage area

	// receive the type of the message
	SDLNet_SocketReady(clientsock);				// poll client socket
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &type, 1) > 0 );		// read the type of the message
	CPPUNIT_ASSERT( type == 'a' );					// assert that the type is actually a gamestate msg

	// read the game state sent
	CPPUNIT_ASSERT( SDLNet_TCP_Recv(clientsock, &ngs, sizeof(netGameState)) > 0 );

	/* make certain that the received gamestate is what we sent */
	// check that the entire board is all tomatoes of state FRESH
	for (int i = 0; i < HEIGHT; i++)
	{
		for (int j = 0; j < WIDTH; j++)
		{
			CPPUNIT_ASSERT( (ngs.board[i][j]).type == TOMATO );		// all tomatoes
			CPPUNIT_ASSERT( (ngs.board[i][j]).state == FRESH );		// all of state FRESH
		}
	}

	// check chef health at 100
	CPPUNIT_ASSERT( ngs.chef.health == 100 );

	// check tip jar size of 50 and currently at 25
	CPPUNIT_ASSERT( ngs.tips.fullamount == 50 );
	CPPUNIT_ASSERT( ngs.tips.tips == 25 );

	// check cursor at (5,5) with alignment of 0 and modifier of 0
	CPPUNIT_ASSERT( ngs.cursor.alignment == 0 );
	CPPUNIT_ASSERT( ngs.cursor.modifier == 0 );
	CPPUNIT_ASSERT( ngs.cursor.pos.x1 == 5 );
	CPPUNIT_ASSERT( ngs.cursor.pos.y1 == 5 );

	// check customer's orders all at 5 onions of state FRESH with times of 10
	for (int cust = 0; cust < NUMCUSTOMERS; cust++)
	{
		for (int orderI = 0; orderI < WIDTH; orderI++)
		{
			CPPUNIT_ASSERT( ngs.customers[cust].order[orderI].type == ONION );
			CPPUNIT_ASSERT( ngs.customers[cust].order[orderI].state == FRESH );
		}

		//CPPUNIT_ASSERT( ngs.customers[cust].timeleft == 10 );
	}
}

void ExampleTestCase::testjoinGame1()
{
	// ------------------------------ Send join using client ---------------------------------------
	MPJoinResponse* jr;
	jr = client->joinGame("Bob");		// player name "Bob"
	CPPUNIT_ASSERT_MESSAGE("A response is built by the client", jr != NULL);

	// ------------------------------ "Server" receives the join -----------------------------------
	// assert that join is received as intended
	char type;				// the type of the message received
	cs_join_attempt ja;		// the join attempt from the client

	// receive the msg type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT_MESSAGE("Receiving Join Game msg type", SDLNet_TCP_Recv(clientsock, &type, 1) > 0);
	CPPUNIT_ASSERT_MESSAGE("Checking for Correct Type", type == 'i' );

	// receive the msg
	CPPUNIT_ASSERT_MESSAGE("Receiving the join msg", SDLNet_TCP_Recv(clientsock, &ja, sizeof(cs_join_attempt)) ==
		sizeof(cs_join_attempt) );
	CPPUNIT_ASSERT_MESSAGE("Checking for correct name", strcmp("Bob", ja.clientName) == 0 );

	// ------------------------------ Client builds a response to engine ---------------------------------------
	// assert that the response is correct

	// Timeout from client before receiving a response
	CPPUNIT_ASSERT_MESSAGE("Test Reponse Type", jr->getType() == 1 );
	CPPUNIT_ASSERT_MESSAGE("Test Response Acceptance", jr->getAcceptance() == false );
	CPPUNIT_ASSERT_MESSAGE("Test Client ID", jr->getClientID() == -1 );
	CPPUNIT_ASSERT_MESSAGE("Test Reason", strcmp((const char*)jr->getReason(), "Timeout") == 0 );

	// free response msg
	delete jr;
}

void ExampleTestCase::testjoinGame2()
{
	// ------------------------------ Send join using client ---------------------------------------
	MPJoinResponse* jr;
	jr = client->joinGame("ThisIsALongNameForSure");	// player name "ThisIsALongNameForSure" (length = 22)
	CPPUNIT_ASSERT_MESSAGE("A response is built by the client", jr != NULL);

	// ------------------------------ "Server" receives the join -----------------------------------
	// assert that join is received as intended
	char type;				// the type of the message received
	cs_join_attempt ja;		// the join attempt from the client

	// receive the msg type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT_MESSAGE("Receiving Join Game msg type", SDLNet_TCP_Recv(clientsock, &type, 1) > 0);
	CPPUNIT_ASSERT_MESSAGE("Checking for Correct Type", type == 'i' );

	// receive the msg
	CPPUNIT_ASSERT_MESSAGE("Receiving the join msg", SDLNet_TCP_Recv(clientsock, &ja, sizeof(cs_join_attempt)) ==
		sizeof(cs_join_attempt) );
	// check for truncated player name (should be: "ThisIsALongNameForSu")
	CPPUNIT_ASSERT_MESSAGE("Checking for correct name", strcmp("ThisIsALongNameForSu", ja.clientName) == 0 );

	// ------------------------------ Client builds a response to engine ---------------------------------------
	// assert that the response is correct

	// Timeout from client before receiving a response
	CPPUNIT_ASSERT_MESSAGE("Test Reponse Type", jr->getType() == 1 );
	CPPUNIT_ASSERT_MESSAGE("Test Response Acceptance", jr->getAcceptance() == false );
	CPPUNIT_ASSERT_MESSAGE("Test Client ID", jr->getClientID() == -1 );
	CPPUNIT_ASSERT_MESSAGE("Test Reason", strcmp((const char*)jr->getReason(), "Timeout") == 0 );

	// free response msg
	delete jr;
}

void ExampleTestCase::testjoinGame3()
{
	/*	The joinGame() method first sends a join request to the server the client is connected to.
	 *	It then waits for a response from the server for a given amount of time.  To test that the
	 *	proper response is being parsed, I simulate the sending of a response to the client first.
	 *	This places the data on the client's socket beforehand.
	 */
	// ------------------------------ Simulate Server Response ---------------------------------------
	char type = 'K';			// join reponse msg type
	sc_join_response scjr;		// join response

	// create a join response msg
	scjr.allowed = true;
	scjr.clientID = 5;
	strcpy(scjr.reason, (const char*)"Allowed");
	scjr.settings.maxPlayers = 8;
	strcpy(scjr.settings.serverName, (const char*)"BustaSandwichd");

	// send the msg
	SDLNet_TCP_Send(clientsock, &type, 1);					// the type of the msg
	SDLNet_TCP_Send(clientsock, &scjr, sizeof(scjr));		// the msg

	// ------------------------------ Send join using client ---------------------------------------
	MPJoinResponse* jr = client->joinGame("BustaSandwich");	// player name "BustaSandwich" (length = 13)
	CPPUNIT_ASSERT_MESSAGE("A response is built by the client", jr != NULL);

	// ------------------------------ "Server" receives the join -----------------------------------
	// assert that join is received as intended
	cs_join_attempt ja;		// the join attempt from the client

	// receive the msg type
	SDLNet_SocketReady(clientsock);			// poll client socket
	CPPUNIT_ASSERT_MESSAGE("Receiving Join Game msg type", SDLNet_TCP_Recv(clientsock, &type, 1) > 0);
	CPPUNIT_ASSERT_MESSAGE("Checking for Correct Type", type == 'i');

	// receive the msg
	CPPUNIT_ASSERT_MESSAGE("Receiving the join msg", SDLNet_TCP_Recv(clientsock, &ja, sizeof(cs_join_attempt)) ==
		sizeof(cs_join_attempt) );
	CPPUNIT_ASSERT_MESSAGE("Checking for correct name", strcmp("BustaSandwich", ja.clientName) == 0 );

	// ------------------------------ Client builds a response to engine ---------------------------------------
	// This is from the data presented to the client in the first step
	// assert that the response is correct

	CPPUNIT_ASSERT_MESSAGE("Test Reponse Type", jr->getType() == 1 );
	CPPUNIT_ASSERT_MESSAGE("Test Response Acceptance", jr->getAcceptance() == true );
	CPPUNIT_ASSERT_MESSAGE("Test Client ID", jr->getClientID() == 5 );
	CPPUNIT_ASSERT_MESSAGE("Test Reason", strcmp((const char*)jr->getReason(), "Allowed") == 0 );
	CPPUNIT_ASSERT_MESSAGE("Correct Server Name", strcmp(client->getServerName(), (const char*)"BustaSandwichd") == 0);

	// free response msg
	delete jr;
}

void ExampleTestCase::testreceiveGameStates()
{
	// yeah right

}

void ExampleTestCase::testreceiveStartGame()
{
	this->testjoinGame3();

	// ------------------------------ Send start game using "server" ---------------------------------------
	char type = 'B';							// start game message type
	SDLNet_TCP_Send(clientsock, &type, 1);		// send the type of the message

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Start Game)", msg->getType() == 202);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPStartGame*)msg;
}

void ExampleTestCase::testreceiveDisconnect()
{
	this->testjoinGame3();

	// ------------------------------ Send disconnect msg using "server" ---------------------------------------
	char type = 'C';				// disconnect msg type
	sc_player_disconnect sc_pd;		// message to send

	// construct a disconnection message
	sc_pd.playerID = 12;		// player 12 disconnected from timeout
	strcpy(sc_pd.discMessage, (const char*)"Timed Out");

	SDLNet_TCP_Send(clientsock, &type, 1);								// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_pd, sizeof(sc_player_disconnect));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ has a msg", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Disconnection)", msg->getType() == 3);
	CPPUNIT_ASSERT_MESSAGE("Correct Player ID disconnected", ((MPPlayerDisconnect*)msg)->getPlayerID() == 12);
	CPPUNIT_ASSERT_MESSAGE("Correct Disconnect Reason", 
		strcmp(((MPPlayerDisconnect*)msg)->getReason(), "Timed Out") == 0 );
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPPlayerDisconnect*)msg;
}

void ExampleTestCase::testreceiveCharSelectMode()
{
	this->testjoinGame3();

	// ------------------------------ Send char select mode using "server" ---------------------------------------
	char type = 'D';							// char select mode message type
	SDLNet_TCP_Send(clientsock, &type, 1);		// send the type of the message

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Character Select Mode)", msg->getType() == 201);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPCharSelectMode*)msg;
}

void ExampleTestCase::testreceivePlayerLost()
{
	this->testjoinGame3();

	// ------------------------------ Send player lost msg using "server" ---------------------------------------
	char type = 'E';			// player lost message type
	sc_player_lost sc_pl;		// player lost message

	// construct a player lost msg
	sc_pl.playerID = 2;		// player 2 lost

	SDLNet_TCP_Send(clientsock, &type, 1);							// send the type of the message
	SDLNet_TCP_Send(clientsock, &sc_pl, sizeof(sc_player_lost));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();
	
	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Player Lost)", msg->getType() == 204);
	CPPUNIT_ASSERT_MESSAGE("Correct player id of lost player", ((MPPlayerLost*)msg)->getPlayerID() == 2);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPPlayerLost*)msg;
}

void ExampleTestCase::testreceiveCharSelection()
{
	this->testjoinGame3();

	// ------------------------------ Send character selection using "server" ----------------------------
	char type = 'F';				// character selection msg type
	sc_character_selected sc_cs;	// character selection msg

	// construct a character selection msg
	sc_cs.selectingPlayer = 7;		// player 7
	sc_cs.character = C7;			// selected character 7

	SDLNet_TCP_Send(clientsock, &type, 1);								// send the type of the message
	SDLNet_TCP_Send(clientsock, &sc_cs, sizeof(sc_character_selected));	// send the message

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Character Selected)", msg->getType() == 101);
	CPPUNIT_ASSERT_MESSAGE("Correct player id of selecting player", ((MPCharSelected*)msg)->getPlayerID() == 7);
	CPPUNIT_ASSERT_MESSAGE("Correct selected characer", ((MPCharSelected*)msg)->getSelectedChar() == C7);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPCharSelected*)msg;
}

void ExampleTestCase::testreceiveCharSelectCursor()
{
	this->testjoinGame3();

	// ------------------------------ Send cursor using "server" ---------------------------------------
	char type = 'G';					// character select cursor message type
	sc_character_select_cursor sc_csc;	// character select message

	// construct a character select message
	sc_csc.playerID = 1;		// player 1's cursor
	sc_csc.cursorLoc.x1 = 6;	// is at (6,2)
	sc_csc.cursorLoc.y1 = 2;

	SDLNet_TCP_Send(clientsock, &type, 1);										// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_csc, sizeof(sc_character_select_cursor));	// send msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Character Selected)", msg->getType() == 102);
	CPPUNIT_ASSERT_MESSAGE("Correct player id of cursor's player", ((MPCharSelectCursor*)msg)->getPlayerID() == 1);
	CPPUNIT_ASSERT_MESSAGE("Correct cursor location",
		(((MPCharSelectCursor*)msg)->getCursor()->getX() == 6) && (((MPCharSelectCursor*)msg)->getCursor()->getY() == 2));
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPCharSelectCursor*)msg;
}

void ExampleTestCase::testreceiveSpecialSandwich()
{
	this->testjoinGame3();

	// ------------------------------ Send special sandwich using "server" ---------------------------------------
	char type = 'H';		// special sandwich message type
	sc_sandwich_send sc_ss;	// special sandwich message

	// construct special sandwich message
	sc_ss.originatingPlayer = 8;	// player 8 sends to player 4 a CONFUSE sandwich
	sc_ss.targetPlayer = 4;
	sc_ss.sandwich = CONFUSE;

	SDLNet_TCP_Send(clientsock, &type, 1);							// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_ss, sizeof(sc_sandwich_send));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is not empty", !(client->getSandwichQ()->isEmpty()) );
	BustaNetInternalMsg* msg = client->getSandwichQ()->pop();		// pop off msg
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty after msg removal", client->getSandwichQ()->isEmpty() );
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Special Sandwich)", msg->getType() == 301);
	CPPUNIT_ASSERT_MESSAGE("Correct sending player id", ((MPSpecialSandwich*)msg)->getSenderID() == 8);
	CPPUNIT_ASSERT_MESSAGE("Correct receiving player id", ((MPSpecialSandwich*)msg)->getReceiverID() == 4);
	CPPUNIT_ASSERT_MESSAGE("Correct send sandwich", ((MPSpecialSandwich*)msg)->getSpecialSandwich() == CONFUSE);

	// delete the message
	delete (MPSpecialSandwich*)msg;
}

void ExampleTestCase::testreceiveNewConnection()
{
	this->testjoinGame3();

	// ------------------------------ Send new connection using "server" ---------------------------------------
	char type = 'I';			// new connection message type
	sc_new_connection sc_nc;	// new connection message

	// construct a new connection message
	sc_nc.playerID = 0;		// player named "Player" has connected with id 0
	strcpy(sc_nc.playerName, (const char*)"Player");

	SDLNet_TCP_Send(clientsock, &type, 1);							// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_nc, sizeof(sc_new_connection));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (New Player)", msg->getType() == 2);
	CPPUNIT_ASSERT_MESSAGE("Correct player id of new player", ((MPNewPlayer*)msg)->getPlayerID() == 0);
	CPPUNIT_ASSERT_MESSAGE("Correct player name", strcmp(((MPNewPlayer*)msg)->getPlayerName(), "Player") == 0 );
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPNewPlayer*)msg;
}

void ExampleTestCase::testreceivePlayerReady()
{
	this->testjoinGame3();

	// ------------------------------ Send Player Ready using "server" ---------------------------------------
	char type = 'L';		// player ready msg type
	sc_player_ready sc_pr;	// player ready msg

	// construct a player ready msg
	sc_pr.playerID = 3;		// player 3 is ready

	SDLNet_TCP_Send(clientsock, &type, 1);							// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_pr, sizeof(sc_player_ready));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Player Ready)", msg->getType() == 103);
	CPPUNIT_ASSERT_MESSAGE("Correct Player ID who is ready", ((MPPlayerReady*)msg)->getClientID() == 3);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPPlayerReady*)msg;
}

void ExampleTestCase::testreceiveError()
{
	this->testjoinGame3();

	// ------------------------------ Send Error using "server" ---------------------------------------
	// send an unknown msg type
	char type = 'm';		// unknown server message (server msgs are capital letters)

	SDLNet_TCP_Send(clientsock, &type, 1);		// send the unknown type

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after msg removal", client->getPriorityQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Error)", msg->getType() == 666);
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty", client->getSandwichQ()->isEmpty());

	// delete the message
	delete (MPError*)msg;
}

void ExampleTestCase::testreceiveManyMsgs()
{
	this->testjoinGame3();

	/*	Compilation of above tests into a single large test.
	 *	Tests the handling of many messages in the socket and in the queues
	 */
	// ------------------------------ Send messages using "server" ---------------------------------------
	char type;			// type of msg

	// send start game
	type = 'B';
	SDLNet_TCP_Send(clientsock, &type, 1);

	// send disconnect
	type = 'C';
	sc_player_disconnect sc_pd;
	sc_pd.playerID = 12;		// player 12 disconnected from timeout
	strcpy(sc_pd.discMessage, (const char*)"Timed Out");
	SDLNet_TCP_Send(clientsock, &type, 1);								// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_pd, sizeof(sc_player_disconnect));	// send the msg

	// send character select mode
	type = 'D';
	SDLNet_TCP_Send(clientsock, &type, 1);

	// send player lost
	type = 'E';
	sc_player_lost sc_pl;
	sc_pl.playerID = 2;		// player 2 lost
	SDLNet_TCP_Send(clientsock, &type, 1);							// send the type of the message
	SDLNet_TCP_Send(clientsock, &sc_pl, sizeof(sc_player_lost));	// send the msg

	// send character selected
	type = 'F';
	sc_character_selected sc_cs;
	sc_cs.selectingPlayer = 7;		// player 7
	sc_cs.character = C7;			//  selected character 7
	SDLNet_TCP_Send(clientsock, &type, 1);								// send the type of the message
	SDLNet_TCP_Send(clientsock, &sc_cs, sizeof(sc_character_selected));	// send the message

	// send character select cursor
	type = 'G';
	sc_character_select_cursor sc_csc;
	sc_csc.playerID = 1;		// player 1's cursor
	sc_csc.cursorLoc.x1 = 6;	//  is at (6,2)
	sc_csc.cursorLoc.y1 = 2;
	SDLNet_TCP_Send(clientsock, &type, 1);										// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_csc, sizeof(sc_character_select_cursor));	// send msg

	// send new connection
	type = 'I';
	sc_new_connection sc_nc;
	sc_nc.playerID = 0;		// player named "Player" has connected with id 0
	strcpy(sc_nc.playerName, (const char*)"Player");
	SDLNet_TCP_Send(clientsock, &type, 1);							// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_nc, sizeof(sc_new_connection));	// send the msg

	// send server settings
	//type = 'J';
	//sc_server_settings sc_ss;
	//strcpy(sc_ss.serverName, (const char*)"BustaSandwichd");	// server name is "BustaSandwichd"
	//SDLNet_TCP_Send(clientsock, &type, 1);								// send msg type
	//SDLNet_TCP_Send(clientsock, &sc_ss, sizeof(sc_server_settings));	// send the msg

	// send error
	type = 'm';		// unknown server message (server msgs are capital letters)
	SDLNet_TCP_Send(clientsock, &type, 1);		// send the unknown type

	/* three special sandwiches */
	// send special sandwich
	type = 'H';
	sc_sandwich_send sc_sandwichS1;
	sc_sandwichS1.originatingPlayer = 2;	// player 2 sends to player 6 a CONFUSE sandwich
	sc_sandwichS1.targetPlayer = 6;
	sc_sandwichS1.sandwich = CONFUSE;
	SDLNet_TCP_Send(clientsock, &type, 1);									// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_sandwichS1, sizeof(sc_sandwich_send));	// send the msg

	// send special sandwich
	sc_sandwich_send sc_sandwichS2;
	sc_sandwichS2.originatingPlayer = 5;	// player 5 sends to player 3 a CONFUSE sandwich
	sc_sandwichS2.targetPlayer = 3;
	sc_sandwichS2.sandwich = CONFUSE;
	SDLNet_TCP_Send(clientsock, &type, 1);									// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_sandwichS2, sizeof(sc_sandwich_send));	// send the msg

	// send special sandwich
	sc_sandwich_send sc_sandwichS3;
	sc_sandwichS3.originatingPlayer = 8;	// player 8 sends to player 4 a CONFUSE sandwich
	sc_sandwichS3.targetPlayer = 4;
	sc_sandwichS3.sandwich = CONFUSE;
	SDLNet_TCP_Send(clientsock, &type, 1);									// send the msg type
	SDLNet_TCP_Send(clientsock, &sc_sandwichS3, sizeof(sc_sandwich_send));	// send the msg

	// ------------------------------ Client Processes Network -----------------------------------
	client->processNet();

	// ------------------------------ Check Message Queues on client -----------------------------------
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is not empty", !(client->getPriorityQ()->isEmpty()));
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is not empty", !(client->getSandwichQ()->isEmpty()));

	BustaNetInternalMsg* msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is still not empty after msg removal", !(client->getPriorityQ()->isEmpty()));
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Start Game)", msg->getType() == 202);
	delete (MPStartGame*)msg;	// delete the msg

	msg = client->getPriorityQ()->pop();	// pull the msg off the queue
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is still not empty after msg removal", !(client->getPriorityQ()->isEmpty()));
	CPPUNIT_ASSERT_MESSAGE("Correct msg type received (Disconnect)", msg->getType() == 3);
	CPPUNIT_ASSERT_MESSAGE("Correct Disconnected Player", ((MPPlayerDisconnect*)msg)->getPlayerID() == 12);
	delete (MPPlayerDisconnect*)msg;	// delete the msg

	// flush the priority Q
	client->getPriorityQ()->flushQ();
	CPPUNIT_ASSERT_MESSAGE("PriorityQ is empty after flush of remaining msgs", client->getPriorityQ()->isEmpty());

	BustaNetInternalMsg* sandwich = client->getSandwichQ()->pop();	// pull the sandwich off the queue
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is still not empty after sandwich removal", !(client->getSandwichQ()->isEmpty()));
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich sender", ((MPSpecialSandwich*)sandwich)->getSenderID() == 2);
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich receiver", ((MPSpecialSandwich*)sandwich)->getReceiverID() == 6);
	CPPUNIT_ASSERT_MESSAGE("Correst Sandwich Type", ((MPSpecialSandwich*)sandwich)->getSpecialSandwich() == CONFUSE);
	delete (MPSpecialSandwich*)sandwich;		// delete the sandwich

	sandwich = client->getSandwichQ()->pop();	// pull the sandwich off the queue
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is still not empty after sandwich removal", !(client->getSandwichQ()->isEmpty()));
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich sender", ((MPSpecialSandwich*)sandwich)->getSenderID() == 5);
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich receiver", ((MPSpecialSandwich*)sandwich)->getReceiverID() == 3);
	CPPUNIT_ASSERT_MESSAGE("Correst Sandwich Type", ((MPSpecialSandwich*)sandwich)->getSpecialSandwich() == CONFUSE);
	delete (MPSpecialSandwich*)sandwich;		// delete the sandwich

	sandwich = client->getSandwichQ()->pop();	// pull the sandwich off the queue
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is empty after processing all", client->getSandwichQ()->isEmpty());
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich sender", ((MPSpecialSandwich*)sandwich)->getSenderID() == 8);
	CPPUNIT_ASSERT_MESSAGE("Correct Sandwich receiver", ((MPSpecialSandwich*)sandwich)->getReceiverID() == 4);
	CPPUNIT_ASSERT_MESSAGE("Correst Sandwich Type", ((MPSpecialSandwich*)sandwich)->getSpecialSandwich() == CONFUSE);
	delete (MPSpecialSandwich*)sandwich;		// delete the sandwich

	client->getSandwichQ()->flushQ();
	CPPUNIT_ASSERT_MESSAGE("SandwichQ is still empty after flushing", client->getSandwichQ()->isEmpty());
}


	// ------------------------------ Send start game using "server" ---------------------------------------
	// ------------------------------ Client Processes Network -----------------------------------
	// ------------------------------ Check Message Queues on client -----------------------------------
