/*	Filename:		BustaEngine.h
 *	Description:	Busta' Sandwich engine module.  Interfaces with the graphics and networking modules.
 *
 */

#ifndef BUSTA_SANDWICH_ENGINE
#define BUSTA_SANDWICH_ENGINE

#include <stdio.h>
#include <string.h>
#include <iostream>
#include <fstream>

#include "../graphics/BustaGraphics.h"
#include "../networking/BustaNetClient.h"
#include "../Models/BustaModelGameState.h"
#include "../Models/BustaModelCompleteSandwich.h"
#include "BustaEngineActionListeners.h"
#include "atltime.h"

using namespace std;

//#define ROTATE SDLK_a
//#define SWAP SDLK_s
//#define DOWN SDLK_DOWN
//#define UP SDLK_UP
//#define RIGHT SDLK_RIGHT
//#define LEFT SDLK_LEFT
//#define QUIT SDLK_ESCAPE
//#define SUBMIT_CUST SDLK_q
//#define SUBMIT_SELF SDLK_w
//#define SUBMIT_OPPONENT SDLK_e

#define HIGHSCORE_PATH "./Resource/hiscores.txt"


/**
 *@enum ScreenState
 *@brief State that the engine is currently in
 *
 * Current state the engine/screen is in
 */
enum ScreenState{MAIN_MENU,OPTIONS,HIGH_SCORES,CHAR_SELECT,SP_GAME,CONNECT,MP_GAME,MP_RANKING,KEY_BINDINGS,SOUNDS,DIFFICULTY,EXIT};


/**
 * @class BustaEngine
 * @brief Busta'Sandwich Engine
 *
 * Busta'Sandwich Engine for single player and networked multiplayer games.
 */
class BustaEngine
{
	friend class ConnectScrnActionListener;
	friend class CharSelectActionListener;
	friend class SPGameActionListener;
	friend class MPGameActionListener;
	friend class MPRankingActionListener;
	friend class HighScoresActionListener;
	friend class KeyBindingsActionListener;

	public:
		

		bool multiplayer;

		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaEngine();

		/**
		 * @brief Destructor
		 *
		 */
		~BustaEngine();

		/**
		 * @brief Gets the current/next screen state
		 *
		 * Gets the engine (screen) state
		 *
		 * @return Returns a ScreenState describing which screen the engine should be in
		 */
		ScreenState getState();

		/**
		 * @brief Sets current/next screen state
		 *
		 * sets the engine (screen) state
		 */
		void setState(ScreenState s);

		/**
		 * @brief Main Menu
		 *
		 * Displays main menu, returns when moving to new menu
		 *
		 */
		void mainMenu();

		/**
		 * @brief Options Menu
		 *
		 * Displays the options menu, returns when moving to new menu
		 * Currently a stub function, sets back to main menu!
		 *
		 */
		void optionsMenu();

		/**
		 * @brief High Scores Screen
		 *
		 * Displays the high scores, returns when moving to main menu
		 *
		 */
		void highScores();

		/**
		 * @brief Character Select Screen Single Player
		 *
		 * Displays the character select screen, allowing the user to choose
		 * a character.  For single player
		 *
		 */
		void characterSelectSP();

		/**
		 * @brief Character Select Screen
		 *
		 * Displays the character select screen, allowing users to choose
		 * characters in a multiplayer game
		 */
		void characterSelectMP();

		/**
		 * @brief Connection Screen
		 *
		 * Displays the connection screen and handles the multiplayer connection
		 * Currently a stub function, sets to CHAR_SELECT!
		 *
		 */
		void connectScreen();

		/**
		 * @brief Key Binding Menu
		 *
		 * Sub-menu under options, allows user to change the key bindings of the game
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void keyBindingMenu();

		/**
		 * @brief Difficulty Screen
		 *
		 * Sub-menu under options, allows user to change the sound options
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void soundsMenu();

		/**
		 * @brief Difficulty Screen
		 *
		 * Sub-menu under options, allows user to change the difficulty of the game
		 * Currently a stub function that sends back to Options menu
		 *
		 */
		void difficultyMenu();


		/**
		 * @brief SP Game
		 *
		 * Displays and runs the game loop (SP or MP), returns when game over for 
		 * single player, when multiplayer is over, or when explicitly exit
		 *
		 */
		void playGame();

		/**
		 * @brief Ranking Screen
		 *
		 * Displays the ranking of the players at the end of a multiplayer game,
		 * takes the names and characters to display in ranked order, with the zeroth
         * array element the 1st player, etc
		 *
		 */
		void displayRankings(char** rankedNames,int* scores, BustaModelCharacter* rankedCharacters);

		/**
		 * @brief Initializes network 
		 *
		 * Initializes the network, returns error messages
		 *
		 * @return Returns a char* describing any errors that occured; returns NULL if initialized properly
		 */
		char* netInit();

		/**
		 * @brief Initializes network connection
		 *
		 * Initializes the network connection, returns error messages
		 *
		 * @return Returns a char* describing any errors that occured while trying to connect to the specified ip;
		 * returns NULL if initialized properly
		 */
		char* connectIP(char* ip);
		/**
		 * @brief moves cursor left
		 *
		 * Moves cursor one space to the left if possible
		 *
		 * @returns a 1 for succesful left movement or a 0 otherwise.
		 */
		int moveLeft(int col_curr, int align_curr);
		/**
		 * @brief moves cursor right
		 *
		 * Moves cursor one space to the right if possible
		 *
		 * @returns a 1 for succesful right movement or a 0 otherwise.
		 */
		int moveRight(int col_curr, int align_curr);
		/**
		 * @brief moves cursor up
		 *
		 * Moves cursor one space up if possible
		 *
		 * @returns a 1 for succesful up movement or a 0 otherwise.
		 */
		int moveUp(int row_curr, int align_curr);
		/**
		 * @brief moves cursor down
		 *
		 * Moves cursor one space down if possible
		 *
		 * @returns a 1 for succesful down movement or a 0 otherwise.
		 */
		int moveDown(int row_curr, int align_curr);
		/**
		 * @brief Swaps two ingredients on the game board
		 *
		 * Swaps the pieces that the movement and alignment cursor point to
		 *
		 * @returns a 1 for succesful swap or a 0 otherwise.
		 */
		int swap(int row_curr, int col_curr, int align_curr); //switch pieces
		/**
		 * @brief rotates alignment cursor clockwise
		 *
		 * Rotates the alignment cursor to next available legal orientation on board in
		 * the clockwise direction.
		 *
		 */
		int rotate(int row_curr, int col_curr, int align_curr); //change axis of rotation
		/**
		 * @brief Customer sandwich submission
		 *
		 * Submits last row of board to customer queue.  If submission doesn't match any order
		 * then chef's eats it otherwise matching customer is removed from queue and health and tip jar
		 * are incremented appropriately.
		 * 
		 * @returns BustaModelCompleteSandwich object representing the submission.
		 */
		BustaModelCompleteSandwich* submit_cus();
		/**
		 * @brief self sandwich submission
		 *
		 * Submits last row of board to player.  If tip jar isn't full than nothing happens otherwis
		 * submitted sandwich is compared against special sandwich types.
		 * 
		 * 
		 * @returns BustaModelCompleteSandwich object representing the submission.
		 */
		BustaModelCompleteSandwich* submit_self();
		/**
		 * @brief opponent sandwich submission
		 *
		 * Submits last row of board to currently selected opponent.  
		 * 
		 * @returns BustaModelCompleteSandwich object representing the submission.
		 */
		BustaModelCompleteSandwich* submit_opponent();
		/**
		 * @brief updates board after sandwich submissions
		 *
		 * Moves all the rows down one and adds a new row of random ingredients on the top.
		 * 
		 */
		void submit_board();
		/**
		 * @brief updates board after penalty sandwich submission
		 *
		 * Moves all the rows down one and adds a new row of random ingredients on the top.
		 * Also decrements from health with amount decremented dependent on freshness of sandwish.
		 * 
		 */
		void submit_penalty();
		BustaModelGameState* getGameState();

	private:
		int ROTATE;
		int	SWAP;
		int DOWN;
		int UP;
		int RIGHT;
		int LEFT;
		int QUIT;
		int SUBMIT_CUST;
		int SUBMIT_SELF;
		int SUBMIT_OPPONENT;

		bool settingkeybinding;
		int key;


		int pause;
		int interval;

		int confusedTime;
		int confusedTimer;
		
		bool networkInitialized;
		bool gameover;
		BustaNetClient* network;
	
		BustaGraphics* graphics;
		BustaModelGameState* gs;
		BustaModelMPGameState* mpgs;
		BustaModelCharacter* mpCharacters;
		int numPlayers;
		int maxPlayers;
		int* rankings;
		int* scores;
		bool setOppGraphics;
		bool showSandwichMenu;
		int myID;
		char myName[21];
		char** mpNames;
		ScreenState state;
		ConnectScrnStates c_state;

		//highscores
		char** HSnames;
		int   HSscores[10];
		char  HSbuffer[20];

		// ------------------- SPECIAL SANDWICH DEFINITIONS -------------------
		FoodType* ssLoseWeight;		// chef loses weight
		FoodType* ssFreshen;		// freshens all ingredients
		FoodType* ssSteal;			// steals tips from opponent
		FoodType* ssEnergyBoost;	// increases timers on all customers
		FoodType* ssGainWeight;		// forces opponent to gain weight
		FoodType* ssSpoil;			// spoils opponent's ingredients
		FoodType* ssJumble;			// jumbles opponent's ingredients
		FoodType* ssConfuse;		// confuses opponent's cursor
		FoodType* ssEnergyZap;		// decrease timers on opponent's customers
		// --------------------------------------------------------------------

		/**
		 * @brief Processes MP network info during game
		 *
		 * Deals with the networking component of the MP game.  Processes
		 * all networking messages.
		 */
		void processMP();

		/**
		 * @brief Handles special sandwich received
		 *
		 * Handles a special sandwich received, applying appropriate action
		 */
		void handleSpecialSandwich(SpecialSandwich s);

		/**
		 * @brief Returns true if x,y coordinate over button area
		 *
		 * Returns true if x,y coordinate over button area
		 */
		bool buttonSelected(Button b, int x, int y);

		/**
		 * @brief Returns next ScreenState
		 *
		 * Takes main screen button which is currently selected, returns state which
		 * game should move to next (main screen, options menu, connection screen, character select screen,etc).
		 */
		ScreenState MainScreenChangeState(MainScrnButton button);

		/**
		 * @brief Returns the chef character that is selected
		 *
		 * Takes a BustaModelPosition*, returns the chef that is selected
		 */
		BustaModelCharacter BustaEngine::selectChef(BustaModelPosition* cursor);

		/**
		 * @brief Checks whether a complete sandwich is a special sandwich
		 *
		 * Checks whether a sandwich is a special sandwich.
		 *
		 * @param sandwich The sandwich to check against for a special sandwich
		 *
		 * @return A special sandwich if the sandwich is of that type
		 */
		SpecialSandwich isSpecialSandwich(BustaModelCompleteSandwich* sandwich);

		/**
		 * @brief Checks two sandwiches against one another for a match
		 *
		 * Checks all ingredients of one sandwich against another sandwich for a match test.  There is 
		 *	a match if the ingredients in one sandwich matches the other.  The ingredients do not have
		 *	to be in the same order.
		 *
		 * @param sandwichOne The first sandwich
		 * @param sandwichTwo The second sandwich
		 *
		 * @return Returns true if the two sandwiches contain the same ingredients and false otherwise.
		 */
		bool sandwichMatch(FoodType* sandwichOne, FoodType* sandwichTwo);


		/**
		 * @brief Sets the sandwich as fresh, normal or rotting
		 *
		 * Checks all ingredients of sandwich if one ingredient is normal entire sandwich is normal,
		 * if one ingredient is rotting entire sandwich is rotting.
		 *
		 * @param sandwich Sandwich whose state is being set.
		 *
		 */
		void setSandwichState(BustaModelCompleteSandwich* sandwich);

		void writeHighScores();

}; 

#endif

