/*
 * BustaGraphicsSprites.h
 * Sprites Class. Loads all the sprites in the game and provides them to BustaGraphics
 *
 * Authors: Julian Chan, Thomas Patecky
 * Created: July 15, 2005
 * Last Modified: 5:41pm, July 25, 2005 
 *
 */
#ifndef BUSTAGRAPHICSSPRITES_H
#define BUSTAGRAPHICSSPRITES_H

#include "../../SDL/include/SDL.h"
#include "Font.h"
#include "../models/BustaModelGameState.h"

//defines what color in the sprite should be considered transparent
#define TRANSPARENT_KEY_R			0
#define TRANSPARENT_KEY_G			255
#define TRANSPARENT_KEY_B			0

//All sprites goes under this folder.
#define PATH_SPRITES				"./Resource/sprites"
#define	MAX_FILE_LENGTH				256

//Food sprites
//** Look inside LoadFoodPieces() for the list of food pieces
#define PATH_FOOD_PIECE_SPRITES		"FoodPieces"
#define PATH_SMALL_FOOD_PIECE_SPRITES	"CustomerOrderPieces"
#define FOOD_PIECE_SIZE				50	
#define SMALL_FOOD_PIECE_SIZE		25
enum FoodFreshness { FOOD_FRESH, FOOD_STALE, FOOD_ROTTEN, FOOD_NUM_FRESH_STATES};

//Universal Game Background
#define	BMP_GAME_BACKGROUND			"background.bmp"

//Chalkboard for special menu
#define BMP_CHALKBOARD				"chalkboard.bmp"

//Main Screen
#define PATH_MAIN_SCREEN			"MainScreen"
#define BMP_MAIN_SCREEN_LOGO		"MainScrnTitle.bmp"

//Connection Screen
#define PATH_CONNECT_SCRN			"ConnectScreen"
#define BMP_RED_LIGHT				"redlight.bmp"
#define BMP_GREEN_LIGHT				"greenlight.bmp"
enum CS_STATUS_LIGHT {GREEN_LIGHT, RED_LIGHT, CS_STATUS_LIGHT_NUM_COLORS};

//Chef animation properties
#define NUM_CHEFS					8
#define ANIM_CHEF_NUM_FRAMES		5
#define CHEF_HEIGHT					164
#define CHEF_WIDTH					115
#define PATH_CHEF_SPRITES			"Chefs"
#define ANIM_CHEF_DELAY				3000	//How long to wait on frame 0 between running animation sequence
#define ANIM_CHEF_FRAMERATE			250		//New frame every 1/4 second
enum ANIM_CHEF_TYPES { CHEF_HEALTHY, CHEF_BLOATED, CHEF_DYING, CHEF_NUM_STATES };

//Customer animation properties
#define NUM_CUSTOMERS				5
#define CUSTOMER_NUM_FRAMES			1
#define ANIM_CUSTOMER_DELAY			5
#define PATH_CUSTOMER_SPRITES		"Patrons"
#define CUSTOMER_HEIGHT				50
#define CUSTOMER_WIDTH				50	
enum ANIM_CUSTOMER_STATES { CUSTOMER_CONTENT, CUSTOMER_PISSED, CUSTOMER_NUM_STATES };


//BMP Font
#define STD_FONT					"stdFont.bmp"
#define CHALK_FONT					"stdFontWhite.bmp"
enum FONTS {STD, CHALK, NUM_FONTS};

//Cursor properties
#define PATH_CURSOR					"Cursor"
#define PATH_SWAP_CURSOR			"SwapCursor"

//Tip Jar
#define PATH_TIP_JAR				"TipJar"
#define TIP_JAR_STAGES				4

/**
 * @class BustaGraphicsSprites
 * @brief contains methods for loading and returning sprites for BustaGraphics
 *
 */
class BustaGraphicsSprites
{
		private:			
				
			//Sprites Storage
			SDL_Surface* food_sprites[NUMFOODPIECES][FOOD_NUM_FRESH_STATES];
			SDL_Surface* food_sprites_small[NUMFOODPIECES];
			SDL_Surface* fonts[2];
			SDL_Surface* cursor_sprites;
			SDL_Surface* swap_cursor_sprites;
			SDL_Surface* chef_sprites[NUM_CHEFS][CHEF_NUM_STATES][ANIM_CHEF_NUM_FRAMES];
			SDL_Surface* main_screen_logo;
			SDL_Surface* tip_jar_sprites[TIP_JAR_STAGES];
			SDL_Surface* customer_sprites[NUM_CUSTOMERS][CUSTOMER_NUM_STATES];			
			SDL_Surface* connect_scrn_status_lights[CS_STATUS_LIGHT_NUM_COLORS];
			SDL_Surface* game_background;
			SDL_Surface* chef_portraits[NUM_CHEFS];
			SDL_Surface* chalkboard;

			SDL_Surface* crusher_sprites[3];


			/**
			 * @brief loads the game background
			 */
			bool LoadGameBackground();

			/**
			 * @brief loads the chalkboard for the special menu
			 */
			bool LoadChalkboard();

			/**
			 * @brief loads all the food sprites in the order specified by the enum FoodType. Also load the small food piece sprites
			 */
			bool LoadFoodPieces();
			
			/**
			 * @brief loads all the small food sprites in the order specified by the enum FoodType. Also load the small food piece sprites
			 */
			bool LoadSmallFoodPieces();
			
			/**
			 * @brief loads a bitmap font
			 */
			bool LoadFonts();
			
			/**
			 * @brief loads the chef sprites
			 */
			bool LoadChefSprites();
			
			/**
			 * @brief loads the sprites for the cursor and swap cursor
			 */
			bool LoadCursorSprites();
	
			/**
			 * @brief loads the logo for main screen
			 */
			bool LoadMainScreenLogo();
			
			/**
			 * @brief loads the tip jar sprites
			 */
			bool LoadTipJarSprites();
			
			/**
			 * @brief loads the tip jar sprites
			 */
			bool LoadCustomerSprites();
		
			bool LoadChefPortraits();

			bool LoadCrusher();

			/**
			 * @brief loads the green light and red light in the connection screen
			 */
			bool LoadConnectScrnStatusLights();

			/**
			 * @brief cuts out the transparent part of a sprite
			 */
			void ApplyTransparency(SDL_Surface *surface);

		public:
			/**
			 * @brief Loads all the sprites in the game
			 */
			BustaGraphicsSprites();
			~BustaGraphicsSprites();
			
			/**
			 * @brief Returns the sprite of the background
			 */
			SDL_Surface* getGameBackground() { return game_background; }

			/**
			 * @brief Returns the sprite of the background
			 */
			SDL_Surface* getChalkboard() { return chalkboard; }

			/**
			 * @brief Returns the sprite of the main screen logo
			 */
			SDL_Surface* getMainScreenLogo() { return main_screen_logo;}
			

			/**
			 * @brief Returns the proper status light for the players in the connection screen
			 * @param light_color which color? you have a choice between RED_LIGHT or GREEN_LIGHT
			 * @return the sprite for a status light, either red or green
			 */
			SDL_Surface* getConnectScrnStatusLight(CS_STATUS_LIGHT light_color);
			
			/**
			 * @brief Returns the propper sprite for a food piece
			 * @param food_piece the food piece to get sprite for
			 * @return sprite for a food type
			 */	
			SDL_Surface* getFoodSprite(BustaModelFoodPiece* food_piece);
			
			/**
			 * @brief Returns a small sprite for a food piece for the customer's order
			 * @param food_piece the food piece to get sprite for
			 * @return sprite for a food type
			 */	
			SDL_Surface* getSmallFoodSprite(BustaModelFoodPiece* food_piece);
			
			/**
			 * @brief Returns a bitmap font used by Sfont
			 * @return a surface for SFont
			 */
			SDL_Surface* getBMPFont(FONTS f) { return fonts[(int)f]; }
			
			/**
			 * @brief Returns the right chef sprite
			 * @param anim_type the type of animation
			 * @param frame which frame of the animation
			 * @return sprite for the chef
			 */
			SDL_Surface* getChefSprites(BustaModelCharacter chef, ANIM_CHEF_TYPES anim_type, int frame);
			
			/**
			 * @brief Returns the right chef sprite
			 * @param state pissed or content
			 * @param customerPortrait which portrait to use
			 * @return sprite for the chef
			 */
			SDL_Surface* getCustomerSprites(ANIM_CUSTOMER_STATES state, int customerPortrait);
			

			/**
			 * @brief Returns the right chef portrait
			 * @param chef the chef
			 * @return sprite for the portrait
			 */
			SDL_Surface* getChefPortraits(BustaModelCharacter chef);

			/**
			 * @brief returns a sprite for the cursor
			 * @return sprite for the cursor
			 */
			SDL_Surface* getCursorSprites() {return cursor_sprites; }
			
			/**
			 * @brief returns a sprite for the swap cursor
			 * @return sprite for the swap cursor
			 */
			SDL_Surface* getSwapCursorSprites() { return swap_cursor_sprites; }

			/**
			 * @brief returns a sprite of the tip jar based on the percent fullness
			 * @param percent_full 100 for a full jar and 50 for a half-full jar
			 * @return sprite for tip jar
			 */
			SDL_Surface* getTipJarSprite(int percent_full);

			SDL_Surface* getCrusher(int state);
};


#endif
