/*	Filename:		BustaModelGameState.h
 *	Description:	Busta' Sandwich GameState model.
 *
 */

#define NULL 0

#define HEIGHT 5
#define WIDTH 5
#define NUMCUSTOMERS 3

#ifndef BUSTA_SANDWICH_GAMESTATE
#define BUSTA_SANDWICH_GAMESTATE

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "BustaModelFoodPiece.h"
#include "BustaModelCustomer.h"
#include "BustaModelTipJar.h"
#include "BustaModelChef.h"
#include "BustaModelCursor.h"

/**
 * @class BustaModelGameState
 * @brief Model representing entire game state
 * 
 * Model representing entire game state, has score, tipjar, chef, customer queue	
 */
class BustaModelGameState
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaModelGameState();

		/**
		 * @brief Destructor
		 *
		 *	Deletes models
		 */
		~BustaModelGameState();


		/**
		 * @brief Gets score
		 *
		 *  Gets the score of the player
		 */
		int getScore();

		/**
		 * @brief Sets score
		 *
		 * Sets the score of the player
		 */
		void setScore(int s);

		/**
		 * @brief Adds customer
		 *
		 * Adds a Customer to the queue, if more than two Customers deletes the oldest one
		 */
		void addCustomer(BustaModelCustomer* c);
		
		/**
		 * @brief Deletes customer
		 *
		 * Deletes the specified customer
		 */
		int removeCustomer(BustaModelCustomer* c);

		/**
		 * @brief Gets customer in queue
		 *
		 * Returns a pointer to the customer in the nth location in the queue
		 */
		BustaModelCustomer*  getCustomer(int n);

		/**
		 * @brief Gets tip jar
		 *
		 * Returns a pointer to the tip jar
		 */
		BustaModelTipJar* getTipJar();

		/**
		 * @brief gets Foodpiece in board
		 *
		 * Returns a pointer to the FoodPiece in the specified row and column
		 */
		BustaModelFoodPiece* getBoard(int i,int j);

		/**
		 * @brief gets the chef
		 *
		 * Returns a pointer to the chef
		 */
		BustaModelChef* getChef();

		/**
		 * @brief Gets cursor
		 *
		 * Returns a pointer to the cursor
		 */
		BustaModelCursor* getCursor();

		/**
		 * @brief Sets board location to specified foodPiece
		 *
		 * For specified board location, removes current foodpiece and sets to FoodPiece fp
		 */
		void setBoard(int i,int j, BustaModelFoodPiece* fp);

		/**
		 * @brief Deletes foodpiece on board
		 *
		 * Deletes the foodpiece in the specified location
		 */
		void removeFromBoard(int i, int j);

		/**
		 * @brief Updates foodpiece on board
		 *
		 * Updates the state of the foodpieces on the board, replaces any necessary.
		 * If u ==0, automatically updates based on time.  If u==1, freshens all ingredients.
		 * If u ==-1, spoils all ingredients.
		 */
		void updateFoodPieces(int u);

		/**
		 * @brief Updates customer queue
		 *
		 * Replaces any customers whose time has run out.  Returns number of 
		 * customers removed from queue.
		 */
		int updateCustomers();

		/**
		 * @brief Jumbles the contents of the board
		 *
		 * Jumbles the order of the ingredients in the board.  The contents are maintained.
		 */
		void jumbleBoard();

	private:
		int score;
		BustaModelFoodPiece* board[HEIGHT][WIDTH];
		BustaModelCustomer* customers[NUMCUSTOMERS];
		BustaModelTipJar* tips;
		BustaModelChef* chef;
		BustaModelCursor* cursor;

		/*
		 * @brief Swaps two foodpieces on the board
		 *
		 * Swaps two foodpieces that are on the board.
		 *
		 * @param first The first foodpiece to swap
		 * @param second The second foodpiece to swap with the first
		 */
		void swapTwo(BustaModelFoodPiece* first, BustaModelFoodPiece* second);

		/*
		 * @brief Swaps two foodpieces on the board
		 *
		 * Swaps two foodpieces on the board.  The foodpieces are denoted by their locations
		 *	on the board.  If any of the coordinates are out of bounds wrt the board the
		 *	function does no operation.
		 *
		 * @param firstX The x-coordinate of the first piece to swap
		 * @param firstY The y-coordinate of the first piece to swap
		 * @param secondX The x-coordinate of the second piece to swap
		 * @param secondY The y-coordinate of the second piece to swap
		 */
		void swapTwo(int firstX, int firstY, int secondX, int secondY);
};

#endif
