/*	Filename:		BustaNetClient.cpp
 *	Description:	Busta' Sandwich client module implementation
 *
 */

#include "BustaNetClient.h"
#include <stdlib.h>

// -----------------------------------------------------------------------------------------------------
// ------------------------------------ Public Functions -----------------------------------------------
// -----------------------------------------------------------------------------------------------------
BustaNetClient::BustaNetClient()
{
	// default client id
	clientID = -1;

	// no server
	serverName = new char[21];
	clientName = new char[21];
	numPlayers = 0;		// no players on server

	/* socket connections */
	tcpsock = NULL;							// no connection to server
	socketset = SDLNet_AllocSocketSet(1);	// socketset for connection to server

	/* Initialize message queues */
	SandwichQ = new BustaNetMsgQ();
	PriorityQ = new BustaNetMsgQ();

	// no multiplayer gamestate of yet
	MPGS = NULL;

	/* PreAllocated Structs for network receives */
	allocateNetworkMemory();
}

BustaNetClient::~BustaNetClient()
{
	// clear sockets
	if (tcpsock != NULL)		// if we were connected
	{
		// disconnect
		this->disconnect();

		// Close socket connection
		SDLNet_TCP_DelSocket(socketset, tcpsock);
		SDLNet_TCP_Close(tcpsock);
		tcpsock = NULL;
	}

	// delete socketset
	SDLNet_FreeSocketSet(socketset);

	// delete message queues
	delete(SandwichQ);
	SandwichQ = NULL;
	delete(PriorityQ);
	PriorityQ = NULL;

	// delete the multiplayer gamestate if it was instantiated (done in disconnect)
	if (MPGS != NULL)
	{
		delete(MPGS);
		MPGS = NULL;
	}

	// delete the rankings array (done in disconnect)
	if (scGameRankings != NULL)
		delete [] scGameRankings;

	// delete the points array (done in disconnect)
	if (scGamePoints != NULL)
		delete [] scGamePoints;

	// delete network structs
	delete scCharacterSelectCursor;
	delete scCharacterSelected;
	delete scJoinResponse;
	delete scNewConnection;
	delete scPlayerDisconnect;
	delete scPlayerLost;
	delete scSandwichSend;
	delete scPlayerReady;
	delete netGSIn;
	delete netGSOut;
	delete csCharacterSelect;
	delete csCursor;
	delete csJoinAttempt;
	delete csOpponentViewSelect;
	delete csSandwichSend;
	if (netOpp != NULL)
	{
		delete [] netOpp;
		netOpp = NULL;
	}

	// Shutdown SDLnet
	SDLNet_Quit();
}

char* BustaNetClient::init()
{
	// Initialize SDLnet
	if (SDLNet_Init() == -1) 
		return SDLNet_GetError();	// On error, return the error message

	return NULL;	// return null on no errors
}

char* BustaNetClient::connect(char* IP, Uint16 serverPort)
{
	// connection status
	char* connected;

	// Resolve the server IP to be stored
	SDLNet_ResolveHost(&serverIP, IP, serverPort);

	// If the address cannot be resolved
	if (serverIP.host == INADDR_NONE) 
	{
		// unable to resolve server address
		connected = "Unable to resolve server address";
	} 
	else
	{
		// Attempt a connection to the server
		tcpsock = SDLNet_TCP_Open(&serverIP);

		// if a connection cannot be made with the server
		if (tcpsock == NULL) 
		{
			// unable to connect to server
			connected = "Unable to connect to server";
		} 
		else 
		{
			// successfully connected
			connected = NULL;
			// add socket to socketset
			SDLNet_TCP_AddSocket(socketset, tcpsock);
		}
	}

	// return connection status
	return connected;
}

void BustaNetClient::disconnect()
{
	// if there is a socket to disconnect from
	if (tcpsock != NULL)
	{
		// send disconnect message
		char type = 'd';						// type
		SDLNet_TCP_Send(tcpsock, &type, 1);		// send disconnect type msg

		// Close socket connection
		SDLNet_TCP_DelSocket(socketset, tcpsock);
		SDLNet_TCP_Close(tcpsock);
		tcpsock = NULL;
	}

	// delete the multiplayer gamestate as it is specific to each server connected to
	if (MPGS != NULL)
	{
		delete MPGS;
		MPGS = NULL;
	}

	// flush the msg queues for a possible new connection
	PriorityQ->flushQ();
	SandwichQ->flushQ();

	// delete the space for network brief gamestate receives
	if (netOpp != NULL)
	{
		delete [] netOpp;
		netOpp = NULL;
	}

	// delete the rankings array
	if (scGameRankings != NULL)
	{
		delete [] scGameRankings;
		scGameRankings = NULL;
	}

	// delete the points array
	if (scGamePoints != NULL)
	{
		delete [] scGamePoints;
		scGamePoints = NULL;
	}

	// disconnected client has id of -1
	clientID = -1;
}

void BustaNetClient::flushSocket()
{
	char data;

	// Loop across entire socket buffer
	while( (SDLNet_CheckSockets(socketset, 0) > 0)	&&	// While there is activity in the socket set AND
			(SDLNet_SocketReady(tcpsock) != 0)			// there is activity on the socket to the server
		)
	{
		// read single bytes until the queue is empty
		SDLNet_TCP_Recv(tcpsock, &data, 1);
	}
}

int BustaNetClient::getClientID()
{
	return clientID;
}

int BustaNetClient::getMaxPlayers()
{
	return maxPlayers;
}

BustaModelMPGameState* BustaNetClient::getMPGS()
{
	return MPGS;
}

int BustaNetClient::getPlayerCount()
{
	return numPlayers;
}

BustaNetMsgQ* BustaNetClient::getPriorityQ()
{
	return PriorityQ;
}

BustaNetMsgQ* BustaNetClient::getSandwichQ()
{
	return SandwichQ;
}

char* BustaNetClient::getServerName()
{
	return serverName;
}

MPJoinResponse* BustaNetClient::joinGame(char* playerName)
{
	MPJoinResponse* mpjs;	// connection response from server

	char type ='i';			// join game network message type
	int nameLen;			// length of player's name
	bool joined;			// whether the join succeeded
	char* message;			// description of join attempt response
	int clientNum;			// the identification number assigned to the client

	// return null if not connected to a server
	if (tcpsock == NULL)
		return NULL;

	/* enclose prefered player name into network message */
	nameLen = strlen((const char*)playerName);	// find the length of the player's name
	if (nameLen > 20)							// if the name is longer than 20 characters
	{
		nameLen = 20;								// truncate the length

		for (int i = 0; i < nameLen; i++)					// Copy player name to enclosing message
		{
			csJoinAttempt->clientName[i] = playerName[i];
			clientName[i] = playerName[i];
		}
		csJoinAttempt->clientName[20] = '\0';				// Add delimiting null
		clientName[20] = '\0';
	}
	else
	{
		strcpy(csJoinAttempt->clientName, (const char*)playerName);
		strcpy(clientName, (const char*)playerName);
	}

	/* send the message */
	SDLNet_TCP_Send(tcpsock, &type, 1);									// send the join game type
	SDLNet_TCP_Send(tcpsock, csJoinAttempt, sizeof(*csJoinAttempt));	// send the msg

	/* Server is to send back joingame response */
	// query socket set for 500ms
	if (SDLNet_CheckSockets(socketset, 500) == 0)		// if no activity in 500ms
	{
		// no response from server
		joined = false;
		message = "Timeout";
		clientNum = -1;
	}
	else
	{
		// if the server has not sent anything back
		if (SDLNet_SocketReady(tcpsock) == 0)
		{
			// no response from server
			joined = false;
			message = "Timeout";
			clientNum = -1;
		}
		else
		{
			// receive the type of the message
			SDLNet_TCP_Recv(tcpsock, &type, 1);

			/* Check the response type of the server */
			if (type != 'K')		// if the type of the response is incorrect
			{
				joined = false;
				message = "Unknown Response From Server";
				clientNum = -1;
			}
			else					// else a correct response is received
			{
				// receive the response from the server
				SDLNet_TCP_Recv(tcpsock, scJoinResponse, sizeof(*scJoinResponse));
				// parse response
				joined = scJoinResponse->allowed;		// whether client was allowed to join
				message = scJoinResponse->reason;		// join response message
				clientNum = scJoinResponse->clientID;	// client identification number
				strcpy(serverName, (const char*)scJoinResponse->settings.serverName);	// save server name
				maxPlayers = scJoinResponse->settings.maxPlayers;						// number of players

				// only setup client if allowed onto server
				if (scJoinResponse->allowed)
				{
					// store client identification number (self)
					clientID = scJoinResponse->clientID;

					// increment the number of players on the server to 1 since self has joined
					numPlayers++;

					/* Initialize the MPGameState */
					MPGS = new BustaModelMPGameState(maxPlayers);
					/* Initialize space to receive brief gamestates */
					netOpp = new netOpponent[maxPlayers];
					/* Initialize the rankings array */
					//scGameRankings = new int[maxPlayers];
					scGameRankings = new int[4];
					/* Initialize the points array */
					scGamePoints = new int[maxPlayers];
					/* Initialize the brief gamestate for self name */
					MPGS->getOpponent(clientID)->setPlayerName(clientName);
					MPGS->getOpponent(clientID)->setValid(true);
				}
			}
		}
	}

	// create the response message to return
	mpjs = new MPJoinResponse(joined, message, clientID);

	return mpjs;
}

bool BustaNetClient::processNet()
{
	int activity = 0;			// measure of activity
	char type;					// the type of the message received
	BustaNetInternalMsg* msg;	// the message to enqueue

	// if there is no mp game state, we are not connected and do not need to process
	if (MPGS == NULL)
		return false;

	// Loop across entire socket buffer
	while( (SDLNet_CheckSockets(socketset, 0) > 0)	&&	// While there is activity in the socket set AND
			(SDLNet_SocketReady(tcpsock) != 0)			// there is activity on the socket to the server
		)
	{
		// Read the type of the message
		SDLNet_TCP_Recv(tcpsock, &type, 1);

		if (type == 'A')			/* Multiplayer Gamestate Update */
		{
			SDLNet_TCP_Recv(tcpsock, netGSIn, sizeof(*netGSIn));				// receive the full gamestate
			SDLNet_TCP_Recv(tcpsock, netOpp, maxPlayers * sizeof(netOpponent));	// receive the brief gamestates
			this->processGSUpdate();		// process the updates
		}
		else if (type == 'B')		/* Start Game Message (sc_start_game) */
		{
			msg = new MPStartGame();		// create internal start message
			PriorityQ->enqueue(msg);		// enqueue in priority Q
		}
		else if (type == 'C')		/* Disconnection Message (sc_player_disconnect) */
		{
			SDLNet_TCP_Recv(tcpsock, scPlayerDisconnect, sizeof(*scPlayerDisconnect));	// receive the msg
			msg = new MPPlayerDisconnect(scPlayerDisconnect->playerID,		// create internal disconnect msg
				scPlayerDisconnect->discMessage);
			PriorityQ->enqueue(msg);		// enqueue the msg on the priority Q

			// decrement count of players
			numPlayers--;

			// track that the brief game state is not valid any longer
			MPGS->getOpponent(scPlayerDisconnect->playerID)->setValid(false);
		}
		else if (type == 'D')		/* Character Select Mode Message (sc_character_select_mode) */
		{
			msg = new MPCharSelectMode();	// create internal character select mode message
			PriorityQ->enqueue(msg);		// enqueue in priority Q
		}
		else if (type == 'E')		/* Player Lost Message (sc_player_lost) */
		{
			SDLNet_TCP_Recv(tcpsock, scPlayerLost, sizeof(*scPlayerLost));	// receive the msg
			msg = new MPPlayerLost(scPlayerLost->playerID);					// create internal lost msg
			PriorityQ->enqueue(msg);		// enqueue in priority Q

			// track that the brief game state is not valid any longer
			MPGS->getOpponent(scPlayerLost->playerID)->setValid(false);
		}
		else if (type == 'F')		/* Character Selection Message (sc_character_selected) */
		{
			SDLNet_TCP_Recv(tcpsock, scCharacterSelected, sizeof(*scCharacterSelected));	// receive the msg
			msg = new MPCharSelected(scCharacterSelected->selectingPlayer,					// create internal msg
				scCharacterSelected->character);
			PriorityQ->enqueue(msg);		// enqueue in priority Q
		}
		else if (type == 'G')		/* Character Selection Cursor (sc_character_select_cursor) */
		{
			SDLNet_TCP_Recv(tcpsock, scCharacterSelectCursor, sizeof(*scCharacterSelectCursor));	// receive the msg
			msg = new MPCharSelectCursor(scCharacterSelectCursor->playerID,							// internal msg
				scCharacterSelectCursor->cursorLoc.x1, scCharacterSelectCursor->cursorLoc.y1);
			PriorityQ->enqueue(msg);			// enqueue in priority Q
		}
		else if (type == 'H')		/* Special Sandwich Sent Message (sc_sandwich_send) */
		{
			SDLNet_TCP_Recv(tcpsock, scSandwichSend, sizeof(*scSandwichSend));		// receive the msg
			msg = new MPSpecialSandwich(scSandwichSend->originatingPlayer,			// internal msg
				scSandwichSend->targetPlayer, scSandwichSend->sandwich);
			SandwichQ->enqueue(msg);		// enqueue in sandwich Q
		}
		else if (type == 'I')		/* New Connection Message (sc_new_connection) */
		{
			SDLNet_TCP_Recv(tcpsock, scNewConnection, sizeof(*scNewConnection));			// receive the msg
			msg = new MPNewPlayer(scNewConnection->playerID, scNewConnection->playerName);	// internal msg
			PriorityQ->enqueue(msg);		// enqueue in priority Q

			// increment count of players
			numPlayers++;

			// track that the brief game state is set to valid
			MPGS->getOpponent(scNewConnection->playerID)->setValid(true);
			// track name of player
			MPGS->getOpponent(scNewConnection->playerID)->setPlayerName(scNewConnection->playerName);
		}
		else if (type == 'K')		/* Server Join Response (sc_join_response) */
		{
			// This Message Is Handled in joinGame()
		}
		else if (type == 'L')		/* Player Ready Message (sc_player_ready) */
		{
			SDLNet_TCP_Recv(tcpsock, scPlayerReady, sizeof(*scPlayerReady));	// receive the msg
			msg = new MPPlayerReady(scPlayerReady->playerID);					// internal msg
			PriorityQ->enqueue(msg);		// enqueue in priority Q
		}
		else if (type == 'M')		/* End Game Message (sc_end_game) */
		{
			SDLNet_TCP_Recv(tcpsock, scGameRankings, maxPlayers*sizeof(int));	// read the rankings array
			SDLNet_TCP_Recv(tcpsock, scGamePoints, maxPlayers*sizeof(int));		// read the points array

			msg = new MPEndGame(maxPlayers, scGameRankings, scGamePoints);
			PriorityQ->enqueue(msg);		// enqueue in priority Q
		}
		else if (type == 'Y')		/* Server Ping Request */
		{
			this->sendPong();			// respond with pong
		}
		else if (type == 'Z')		/* Server Pong */
		{
			// nothing to do
		}
		else						/* Undefined Message Type */
		{
			msg = new MPError();			// create internal error msg
			PriorityQ->enqueue(msg);		// enqueue the error msg on the priority Q
		}

		// keep track of activity from server
		activity++;
	}

	// if no activity from server, notify on priority Q
	if (activity == 0)
		return false;				// no activity
	else
		return true;		// activity

}

void BustaNetClient::sendCharSelect(BustaModelCharacter character)
{
	char type = 'b';		// the type of the message to send

	csCharacterSelect->character = character;	// enclose the selection in the msg

	SDLNet_TCP_Send(tcpsock, &type, 1);											// Send the type of the msg
	SDLNet_TCP_Send(tcpsock, csCharacterSelect, sizeof(*csCharacterSelect));	// send the selection
}

void BustaNetClient::sendCursor(BustaModelPosition* cursor)
{
	char type = 'c';	// the type of the message to send

	// enclose the cursor location in the net message
	csCursor->cursorLoc.x1 = cursor->getX();
	csCursor->cursorLoc.y1 = cursor->getY();

	SDLNet_TCP_Send(tcpsock, &type, 1);						// send the type of the message
	SDLNet_TCP_Send(tcpsock, csCursor, sizeof(*csCursor));	// send the cursor
}

void BustaNetClient::sendGameOver()
{
	char type = 'e';						// game over msg type
	SDLNet_TCP_Send(tcpsock, &type, 1);		// send the type of the message
}

void BustaNetClient::sendOpponentViewReq(int opponentID)
{
	char type = 'f';				// type of message to send
	
	// enclose selection in message
	csOpponentViewSelect->opponentID = opponentID;

	SDLNet_TCP_Send(tcpsock, &type, 1);												// send the msg type
	SDLNet_TCP_Send(tcpsock, csOpponentViewSelect, sizeof(*csOpponentViewSelect));	// send the msg

	// track the network identification of the requested player
	if (MPGS != NULL)
		MPGS->setFullViewID(opponentID);
}

void BustaNetClient::sendPing()
{
	if (tcpsock != NULL)
	{
		char type = 'y';						// ping msg type
		SDLNet_TCP_Send(tcpsock, &type, 1);		// send the msg
	}
}

void BustaNetClient::sendReady()
{
	if (tcpsock != NULL)
	{
		char type = 'g';						// ready msg type
		SDLNet_TCP_Send(tcpsock, &type, 1);		// send the type of the message
	}
}

void BustaNetClient::sendSandwich(int opponentID, SpecialSandwich sandwich)
{
	if (tcpsock != NULL)
	{
		char type = 'h';		// type of the message to send

		// enclose the the sandwich in the message
		csSandwichSend->targetPlayer = opponentID;
		csSandwichSend->sandwich = sandwich;

		SDLNet_TCP_Send(tcpsock, &type, 1);									// send the type of the message
		SDLNet_TCP_Send(tcpsock, csSandwichSend, sizeof(*csSandwichSend));	// send the msg
	}
}

void BustaNetClient::sendState(BustaModelGameState* gs)
{
	int i, j;				// array iterators
	char type = 'a';		// gamestate msg type

	// don't send if the client is not connected to a server
	if (tcpsock == NULL)
		return;

	// send own client id
	netGSOut->clientID = clientID;

	//score
	netGSOut->score = gs->getScore(); 

	//game board
	for(i = 0; i < HEIGHT; i++)
	{
		for(j = 0; j < WIDTH; j++)
		{
			netGSOut->board[i][j].state = (gs->getBoard(i, j))->getState();
			netGSOut->board[i][j].type = (gs->getBoard(i, j))->getType();
		}
	}

	//customer queue
	for(i = 0; i < NUMCUSTOMERS; i++)
	{
		for(j = 0; j < WIDTH; j++)
		{
			netGSOut->customers[i].order[j].state = ((gs->getCustomer(i))->getOrder())[j]->getState();
			netGSOut->customers[i].order[j].type = ((gs->getCustomer(i))->getOrder())[j]->getType();
		}

		// customer order time
		netGSOut->customers[i].totalTime = (gs->getCustomer(i))->getAllottedTime();	// total time
		netGSOut->customers[i].timeleft = (gs->getCustomer(i))->getTime();			// time left
	}

	//tip jar
	netGSOut->tips.tips = (gs->getTipJar())->getTips();
	netGSOut->tips.fullamount = (gs->getTipJar())->getSize();

	//chef
	netGSOut->chef.health = (gs->getChef())->getHealth();

	//cursor
	netGSOut->cursor.alignment	= (gs->getCursor())->getAlignment();
	netGSOut->cursor.modifier	= (gs->getCursor())->getModifier();
	netGSOut->cursor.pos.x1		= ((gs->getCursor())->getPosition())->getX();
	netGSOut->cursor.pos.y1		= ((gs->getCursor())->getPosition())->getY();

	SDLNet_TCP_Send(tcpsock, &type, 1);						// send the net gamestate type
	SDLNet_TCP_Send(tcpsock, netGSOut, sizeof(*netGSOut));	// send the net gamestate
}

// -----------------------------------------------------------------------------------------------------
// ----------------------------------- Private Functions -----------------------------------------------
// -----------------------------------------------------------------------------------------------------
void BustaNetClient::allocateNetworkMemory()
{
	/* messages from server */
	scCharacterSelectCursor = new sc_character_select_cursor;
	scCharacterSelected = new sc_character_selected;
	scJoinResponse = new sc_join_response;
	scNewConnection = new sc_new_connection;
	scPlayerDisconnect = new sc_player_disconnect;
	scPlayerLost = new sc_player_lost;
	scSandwichSend = new sc_sandwich_send;
	//scServerSettings = new sc_server_settings;
	scPlayerReady = new sc_player_ready;
	netGSIn = new netGameState;
	netOpp = NULL;		// unknown size at this time
	scGameRankings = NULL;
	scGamePoints = NULL;

	/* messages to server */
	netGSOut = new netGameState;
	csCharacterSelect = new cs_character_select;
	csCursor = new cs_character_cursor;
	csJoinAttempt = new cs_join_attempt;
	csOpponentViewSelect = new cs_opponent_view_select;
	csSandwichSend = new cs_sandwich_send;
}

void BustaNetClient::processGSUpdate()
{
	/* Brief Game States */
	for (int i = 0; i < maxPlayers; i++)
	{
		// if it's a valid opponent, do the update
		if (MPGS->getOpponent(i)->isValid())
		{
			MPGS->getOpponent(i)->setHealth(netOpp[i].health);						// update health
			MPGS->getOpponent(i)->getTipJar()->setSize(netOpp[i].tips.fullamount);	// update tipjar size
			MPGS->getOpponent(i)->getTipJar()->setTips(netOpp[i].tips.tips);		// update tip amount
		}
	}

	/* Full Game State */

	// if the server sent a full gamestate for a different client that requested
	if (MPGS->getFullViewID() != netGSIn->clientID)
	{
		//rerequest the correct client's full gamestate
		this->sendOpponentViewReq(MPGS->getFullViewID());
		// return without updating
		return;
	}

	// Handle score
	MPGS->getGameState()->setScore(netGSIn->score);

	// Handle chef
	MPGS->getGameState()->getChef()->setHealth(netGSIn->chef.health);		// health

	// Handle tips
	MPGS->getGameState()->getTipJar()->setSize(netGSIn->tips.fullamount);	// tipjar size
	MPGS->getGameState()->getTipJar()->setTips(netGSIn->tips.tips);			// tip amount

	// Handle cursor
	MPGS->getGameState()->getCursor()->setPosition(netGSIn->cursor.pos.x1, netGSIn->cursor.pos.y1);	// position
	MPGS->getGameState()->getCursor()->setModifier(netGSIn->cursor.modifier);						// modifier
	MPGS->getGameState()->getCursor()->setAlignment(netGSIn->cursor.alignment);						// alignment

	// Handle the entire board
	for (int r = 0; r < HEIGHT; r++)
	{
		for (int c = 0; c < WIDTH; c++)
		{
			// If the type of the foodpiece is different, update it
			if (MPGS->getGameState()->getBoard(r,c)->getType() != netGSIn->board[r][c].type)
				MPGS->getGameState()->getBoard(r,c)->setType(netGSIn->board[r][c].type);
			// update the state of the foodpiece
			MPGS->getGameState()->getBoard(r,c)->setState(netGSIn->board[r][c].state);
		}
	}

	// Handle customers
	for (int i = 0; i < NUMCUSTOMERS; i++)
	{
		// handle the order
		for (int j = 0; j < WIDTH; j++)
		{
			MPGS->getGameState()->getCustomer(i)->getOrder(j)->setType(netGSIn->customers[i].order[j].type);	// type
			MPGS->getGameState()->getCustomer(i)->getOrder(j)->setState(netGSIn->customers[i].order[j].state);	// state
		}

		// handle customer time
		MPGS->getGameState()->getCustomer(i)->setAllottedTime(netGSIn->customers[i].totalTime);		// total time
		MPGS->getGameState()->getCustomer(i)->setTimeLeft(netGSIn->customers[i].timeleft);			// remaining time
	}
}

void BustaNetClient::sendPong()
{
	if (tcpsock != NULL)
	{
		char type = 'z';						// pong msg type
		SDLNet_TCP_Send(tcpsock, &type, 1);		// send pong msg
	}
}
