/*	Filename:		BustaNetClient.h
 *	Description:	Busta' Sandwich client module.  Interfaces with game engine and server
 *
 */

#ifndef BUSTA_SANDWICH_CLIENT
#define BUSTA_SANDWICH_CLIENT

#include "../../SDL/include/SDL_net.h"
#include "../../BustaSandwichd/BustaNetMessages.h"
#include "BustaNetInternal.h"
#include "BustaNetMsgQ.h"
#include "../models/BustaModelGameState.h"
#include "../models/BustaModelMPGameState.h"
#include "../models/BustaModelCharacter.h"
#include "../models/BustaModelSpecialSandwich.h"
#include <string.h>

/**
 * @class BustaNetClient
 * @brief Client network interface to Busta'Sandwich
 *
 * Busta'Sandwich Client for networked multiplayer games.
 */
class BustaNetClient
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables.  A multiplayer gamestate is NOT constructed at this point.  Construction
		 *	of the MPGameState is left to the joinGame() method; joinGame() receives the required information
		 *	about the server from the server which allows it to know how to construct the MPGameState.
		 *
		 * @see joinGame()
		 */
		BustaNetClient();

		/**
		 * @brief Destructor
		 *
		 * Close any open connections and shutdown SDLNet
		 */
		~BustaNetClient();

		/**
		 * @brief Initializes the client and prepares it for connection
		 *
		 * Initializes SDLNet.  Relays any error message back.
		 *
		 * @return Returns a cstring containing an error message if SDLNet cannot be initialized.
		 *	On no error, null is returned.
		 */
		char* init();

		/**
		 * @brief Connects to a server using a given port
		 *
		 * The client initializes a TCP connection with the server on the specified port.
		 *
		 * @param IP The IP address of the server to connect to.
		 * @param serverPort The port of the server to connect to.
		 *
		 * @return A cstring containing information about the connection.  Returns NULL on successful
		 *	connection to the server, or a cstring about connection failure on an unsuccessful connection.
		 */
		char* connect(char* IP, Uint16 serverPort);

		/**
		 * @brief Disconnects from the server
		 *
		 * The connection between the server and client is disconnected.  Subsequent connections may
		 *	be made using the client.  The multiplayer gamestate is also freed from memory.
		 */
		void disconnect();

		/**
		 * @brief Flushes the socket connection to the server of all data
		 *
		 * Empties the socket queue of all data.  You may want to do this if you suspect an error in
		 *	parsing the data from the socket.  For instance, if the client is 1 byte off in pulling
		 *	data from the socket, all the data will be corrupt.  It may be better to flush the entire
		 *	socket and wait for new messages from the server altogether.
		 */
		void flushSocket();

		/**
		 * @brief Returns the network identification number of the client
		 *
		 * All clients in a multiplayer game are given a unique network identification number.  This is 
		 *	the means by which clients can distinguish other clients from one another.  This method
		 *	returns the client identification number assigned to this client.  By default it is set to -1,
		 *	and it is reset to -1 when disconnected from a server.  When the client has successfully
		 *	joined a network game the server will assign the client a new number.
		 *
		 * @return The network identification number of the client.  Is -1 when the client is not a part
		 *	of a multiplayer game.
		 */
		int getClientID();

		/**
		 * @brief Returns the maximum number of players the server allows
		 *
		 * Returns the number of players that the server allowed as a maximum.
		 *
		 * @return The maximum number of players allowed on the server
		 */
		int getMaxPlayers();

		/**
		 * @brief Returns game state information about opponent players
		 *
		 * Returns a pointer to a BustaModelMPGameState containing information about opponent players.  The
		 *	MPGameState is only instantiated after the client has properly connected and joined a game on
		 *	a server.
		 *
		 * @return A BustaModelMPGameState containing opponent game states.  NULL is returned if the multiplayer
		 *	gamestate has not been yet instantiated.
		 *
		 * @see BustaModelMPGameState
		 */
		BustaModelMPGameState* getMPGS();

		/**
		 * @brief Returns the number of players on the server
		 *
		 * Returns the number of players that are on the server that the client is connected to.  This is the
		 *	number of other players on the server, which does not include the client in the count.
		 *
		 * @return The number of players on the server other than this client
		 */
		int getPlayerCount();

		/**
		 * @brief Returns the server priority message queue
		 *
		 * Returns the queue of messages from the server.  These messages do not cover special sandwiches or
		 *	game states.
		 *
		 * @return A BustaNetMsgQ pointer to a server message queue
		 *
		 * @see BustaNetMsgQ
		 */
		BustaNetMsgQ* getPriorityQ();

		/**
		 * @brief Returns the special sandwich queue
		 *
		 * Returns the queue of special sandwiches sent between players in a multiplayer game.
		 *
		 * @return A BustaNetMsgQ pointer to a special sandwich message queue
		 *
		 * @see BustaNetMsgQ
		 */
		BustaNetMsgQ* getSandwichQ();

		/**
		 * @brief Returns the name of the server connected to
		 *
		 * Returns the name of the server the client is currently connected to.
		 *
		 * @return The name of the server
		 */
		char* getServerName();

		/**
		 * @brief Joins the server multiplayer game
		 *
		 * This method attempts to connect the client to the multiplayer game hosted by the server.  The
		 *	client should be connected to the server before running this method.  The multiplayer gamestate
		 *	is instantiated after calling this method if the join was successful.
		 *
		 * @param playerName The name the player wishes to use for the multiplayer game.  Names longer than
		 *	20 characters will be truncated.
		 *
		 * @return A MPJoinResponse with the server's response to the client's connect attempt.  Returns 
		 *	null if the client is not already connected to the server.
		 *
		 * @see connect()
		 */
		MPJoinResponse* joinGame(char* playerName);

		/**
		 * @brief Handles all incoming network data
		 *
		 * Processes all data currently queued on the TCP socket.  The data is partitioned out into
		 *	three message containers.  Each of these three message containers is accessible through
		 *	an accompanying accessor method.
		 *
		 * @return Returns true if there was activity from the server to process and false otherwise
		 *
		 * @see getMPGS()
		 * @see getPriorityQ()
		 * @see getSandwichQ()
		 */
		bool processNet();

		/**
		 * @brief Sends character selection of player
		 *
		 * Sends the character selection by the player to the server to be sent to all connected
		 *	players.
		 *
		 * @param character The selected character
		 */
		void sendCharSelect(BustaModelCharacter character);

		/**
		 * @brief Sends current character selection cursor position of player
		 *
		 * Sends the current position of the player's cursor during multiplayer character selection.
		 *
		 * @param cursor The cursor position of the player's character selection cursor
		 */
		void sendCursor(BustaModelPosition* cursor);

		/**
		 * @brief Notifies the server that the player has lost the game
		 *
		 * A message is sent to the server to notify it that the player has lost in the current
		 *	multiplayer game.
		 */
		void sendGameOver();

		/**
		 * @brief Tells the server which opponent the player wishes to view
		 *
		 * A message is sent to the server to tell it which opponent the player wishes to have a
		 *	detailed view of.  The detailed view contains information to display the entire
		 *	screen of the opponent rather than limited brief data.
		 *
		 * @param opponentID The identification of the opponent whom the player wishes to view
		 */
		void sendOpponentViewReq(int opponentID);

		/**
		 * @brief Sends a simple response request to the server
		 *
		 * Sends a response request to the server.  This is useful when the client is unsure whether
		 *	or not the server is still responding to the client or connected at all.  The server
		 *	follows up with a 'Pong' response.
		 */
		void sendPing();

		/**
		 * @brief Sends a ready message to the server
		 *
		 * Sending a ready message to the server indicates that the client is ready to begin the
		 *	multiplayer game.  The ready state is visible to all other clients and is used to determine
		 *	when the server should transition from the accepting connections state.
		 */
		void sendReady();

		/**
		 * @brief Sends a spcial sandwich across the network
		 *
		 * A special sandwich is sent from the player to an opposing player.
		 *
		 * @param opponentID The identification of the opponent to whom the player wishes to send the sandwich
		 * @param sandwich The special sandwich to send
		 */
		void sendSandwich(int opponentID, SpecialSandwich sandwich);

		/**
		 * @brief Sends player's game state to the server
		 *
		 * A BustaModelGameState is converted into structs without pointers to form a packet
		 *  to send to the server.  It makes two sends, a packet identifier message, and the
		 *  netGameState data packet.
		 *
		 * @param gs The game state of the player
		 */
		void sendState(BustaModelGameState* gs);

	private:
		int clientID;						// Network Identification number of the client
		IPaddress serverIP;					// IP address of the game server
		char* clientName;					// client's own name
		char* serverName;					// server name
		int maxPlayers;						// maximum number of players on the server
		int numPlayers;						// the number of players on the server
		BustaModelMPGameState* MPGS;		// Opponent players' game states
		BustaNetMsgQ* PriorityQ;			// Queue of server messages
		BustaNetMsgQ* SandwichQ;			// Queue of special sandwich sends
		SDLNet_SocketSet socketset;			// Socketset containing server connection socket
		TCPsocket tcpsock;					// Socket connection to server

		/** PreAllocated structs for network receiving **/
		/* msgs from the server */
		sc_character_selected* scCharacterSelected;
		//sc_character_select_mode* scCharacterSelectMode;		currently empty
		sc_character_select_cursor* scCharacterSelectCursor;
		sc_new_connection* scNewConnection;
		sc_player_disconnect* scPlayerDisconnect;
		sc_player_lost* scPlayerLost;
		sc_sandwich_send* scSandwichSend;
		//sc_server_settings* scServerSettings;
		//sc_start_game* scStartGame;							currently empty
		sc_join_response* scJoinResponse;
		sc_player_ready* scPlayerReady;
		netGameState* netGSIn;
		netOpponent* netOpp;
		int* scGameRankings;
		int* scGamePoints;
		/* msgs to the server */
		netGameState* netGSOut;
		cs_character_select* csCharacterSelect;
		cs_character_cursor* csCursor;
		//cs_disconnect* csDisconnect;							currently empty
		//cs_gameover* csGameOver;								currently empty
		cs_join_attempt* csJoinAttempt;
		cs_opponent_view_select* csOpponentViewSelect;
		//cs_ready* csReady;									currently empty
		cs_sandwich_send* csSandwichSend;

		/* @brief Preallocates memory for network traffic
		 *
		 * Preallocates memory for network send and receive structs.  Good for one-time
		 *	allocation.  Initializes structs in memory used for sending packets to the
		 *	server as well as receiving from the network into memory.
		 */
		void allocateNetworkMemory();

		/*
		 * @brief Processes the update of gamestates from the server
		 *
		 * Processes the full gamestate and brief gamestate from the server's update.
		 */
		void processGSUpdate();

		/*
		 * @brief Sends a 'pong' to the server
		 *
		 * Sends a 'pong' to the server.  Useful for anti-idle techniques.  Often used when
		 *	the server has specifically requested a 'pong' response from the client.  This is
		 *	done when the server sends a 'ping' to the client.
		 */
		void sendPong();
};

#endif
