/*	Filename:		BustaNetMessages.h
 *	Description:	Contains definitions of all net messages sent across the network.
 *
 */

#ifndef BUSTA_NET_MESSAGES_H
#define BUSTA_NET_MESSAGES_H

#include "../BustaSandwich/models/BustaModelCharacter.h"
#include "../BustaSandwich/models/BustaModelPosition.h"
#include "../BustaSandwich/models/BustaModelSpecialSandwich.h"
#include "../BustaSandwich/models/BustaModelFoodPiece.h"

// -----------------------------------------------------------------------------------------------
// ------------------------------- GLOBAL MESSAGES -----------------------------------------------
// -----------------------------------------------------------------------------------------------

/**
 * @struct netPosition BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelPosition data
 *
 * This struct contains all of the data elements of a BustaModelPosition instance,
 *  which is an x and y position.
 */
struct netPosition
{
	int x1;					/**< Cursor position on the x axis */
	int y1;					/**< Cursor position on the y axis */
};

/**
 * @struct netCursor BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelCursor data
 *
 * This struct contains all of the data elements of a BustaModelCursor instance,
 *  which is an alignment, a modifier, and an instance of a netPosition struct.
 */
struct netCursor
{
	netPosition pos;		/**< A netPosition  */
	int alignment;			/**< The cursor alignment */
	int modifier;			/**< The cursor modifier (confused mode) */
};

/**
 * @struct netFoodPiece BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelFoodPiece data
 *
 * This struct contains all of the data elements of a BustaModelFoodPiece instance,
 *  which is a state and type of the piece.
 */
struct netFoodPiece
{
	FoodType type;			/**< The type of food piece */
	FoodState state;		/**< The state (freshness) of a food piece */
};

/**
 * @struct netCustomer BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelCustomer data
 *
 * This struct contains all of the data elements of a BustaModelCustomer instance,
 *  which is life lelf on an order and an array of netFoodPiece elements representing
 *  that order.
 */
struct netCustomer
{
	int timeleft;			/**< The time left on an order */
	int totalTime;			/**< The total amount of time that was given to fulfill the order */
	netFoodPiece order[5];	/**< The array of food pieces that makes up an order sandwich */
};

/**
 * @struct netTipJar BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelTipJar data
 *
 * This struct contains all of the data elements of a BustaModelTipJar instance,
 *  which is the amount of tips and the amount it takes to reach a full jar.
 */
struct netTipJar
{
	int tips;				/**< The amount of tips you have */
	int fullamount;			/**< The amount when you will reach full tips  */
};

/**
 * @struct netChef BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelChef data
 *
 * This struct contains all of the data elements of a BustaModelChef instance,
 *  which is a health value.
 */
struct netChef 
{
	int health;				/**< The health index of the chef */
};

/**
 * @struct netGameState BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelGameState data
 *
 * This struct contains all of the data elements of a BustaModelGameState instance,
 *  which is a score, an array of netFoodPieces for the board, an array of netCustomers,
 *  and an instance of netTipJar, netChef, and netCursor.
 */
struct netGameState
{
	int clientID;					/**< The network Identification number of the client whose gamestate this is */
	int score;						/**< The current score */
	netFoodPiece board[5][5];		/**< An array of netFoodPieces representing the board */
	netCustomer customers[3];		/**< An array of netCustomers representing your customer queue */
	netTipJar tips;					/**< An instance of netTipJar */
	netChef chef;					/**< An instance of netChef */
	netCursor cursor;				/**< An instance of netCursor */
};	

/**
 * @struct netOpponent BustaNetMessages.h
 *
 * @brief A struct encapsulation of BustaModelOpponent data
 *
 * This struct contains all of the data elements of a BustaModelOpponent instance,
 *  which is a character identifier, health index and an instance of netTipJar.
 */
struct netOpponent
{
	int clientID;					/**< The network identification number of the client whose brief state this is */
	int health;						/**< The health of that player */
	netTipJar tips;					/**< The amount of tips that player has */
};

// -----------------------------------------------------------------------------------------------
// ------------------------------- CLIENT MESSAGES -----------------------------------------------
// -----------------------------------------------------------------------------------------------

/**
 * @struct cs_join_attempt BustaNetMessages.h
 *
 * @brief (i) - Client message to server attempting to join server game
 *
 * The message contains the client's preferred multiplayer game name.  The message acts as a notifier
 *	to the server that the client wishes to join the game.  The player's name is limited to 20 characters.
 */
struct cs_join_attempt
{
	char clientName[21];	/**< The name of the client's player */
};

/**
 * @struct cs_character_select BustaNetMessages.h
 *
 * @brief (b) - Client sends a character selection
 *
 * This message is sent from a client to the server detailing the character selected by the player.
 *	Each player selects a character chef for a multiplayer game.  Once a selection is made, this
 *	message is sent to the server dictating which character was selected by the client.
 */
struct cs_character_select
{
	BustaModelCharacter character;		/**< The character the player has selected */
};

/**
 * @struct cs_character_cursor BustaNetMessages.h
 *
 * @brief (c) - Client moves the cursor during character selection
 *
 * The message contains the location of the cursor when the client is selecting a multiplayer
 *	character.  This notifies the server of the client's cursor position when selecting takes place.
 */
struct cs_character_cursor
{
	netPosition cursorLoc;		/**< The location of the cursor */
};

/**
 * @struct cs_disconnect BustaNetMessages.h
 *
 * @brief (d) - Client disconnects from the server
 *
 * This message is a notification to the server that the client is disconnecting.
 */
struct cs_disconnect
{

};

/**
 * @struct cs_gameover BustaNetMessages.h
 *
 * @brief (e) - Client sends a notification to the server that the player has lost
 *
 * This message is a notification to the server that the player has lost in the current multiplayer
 *	game.
 */
struct cs_gameover
{

};

/**
 * @struct cs_opponent_view_select BustaNetMessages.h
 *
 * @brief (f) - Client requests to be sent full game state information about a particular opponent
 *
 * The message contains the identification number of the opponent the client wishes to see full
 *	detail of.  A client who sends this message will get full game state updates for the specified
 *	opponent.
 */
struct cs_opponent_view_select
{
	int opponentID;		/**< The identification number of the specified opponent */
};

/**
 * @struct cs_ready BustaNetMessages.h
 *
 * @brief (g) - Client notifies the server that player is ready to transition out of the connect state
 *
 * A client sends this message when the player is ready to move to character selection.  Sending this
 *	message signifies that the client wishes to begin the game along with everyone else already
 *	connected to the server.
 */
struct cs_ready
{

};

/**
 * @struct cs_sandwich_send BustaNetMessages.h
 *
 * @brief (h) - Client sends a sandwich to an opponent player
 *
 * This message is sent to the server when a player sends a special sandwich to an opponent.
 */
struct cs_sandwich_send
{
	int targetPlayer;			/**< The identification number of the opponent client */
	SpecialSandwich sandwich;	/**< The special sandwich to be sent */
};

/**
 * @struct cs_ping BustaNetMessages.h
 *
 * @brief (y) - Client requests a response from the server
 *
 * A client sends this message to confirm that the server is still responding.  This is followed
 *	by a 'pong' from the server signaling that the connection between the client and server
 *	is still available.
 */
struct cs_ping
{

};

/**
 * @struct cs_pong BustaNetMessages.h
 *
 * @brief (z) - Client responds to a server ping request
 *
 * A client sends this message to the server in response to a server's 'ping' request.  This
 *	signals to the server that the client is still connected to it.
 */
struct cs_pong
{

};

// -----------------------------------------------------------------------------------------------
// ------------------------------- SERVER MESSAGES -----------------------------------------------
// -----------------------------------------------------------------------------------------------

/**
 * @struct sc_character_selected BustaNetMessages.h
 *
 * @brief (F) - Server notification to client of character selection
 *
 * The server sends this message to a client when another client has made a character selection.
 *	This allows the client to know when and what character was selected by an opponent.
 */
struct sc_character_selected
{
	int selectingPlayer;			/**< The identification number of the client who has selected */
	BustaModelCharacter character;	/**< The character selected by the player */
};

/**
 * @struct sc_character_select_mode BustaNetMessages.h
 *
 * @brief (D) - Server notification to client to begin selecting a character
 *
 * This message is sent from the server to a client to notify the client it is time to make
 *	a multiplayer character selection.
 */
struct sc_character_select_mode
{

};

/**
 * @struct sc_character_select_cursor BustaNetMessages.h
 *
 * @brief (G) - Server notification of a client's cursor location in character select mode
 *
 * The server uses this message when all characters are in character selection mode.  A
 *	client may see the cursors of all other opponents when character selections are made.
 *	This message tells a client the location of another client's cursor.
 */
struct sc_character_select_cursor
{
	int playerID;				/**< The identification number of the cursor's player */
	netPosition cursorLoc;		/**< The location of the player's cursor */
};

/**
 * @struct sc_new_connection BustaNetMessages.h
 *
 * @brief (I) - Server notification of a newly connected client
 *
 * This message is sent from the server to a notify a client that a new client has
 *	connected to the server and has joined the game.
 */
struct sc_new_connection
{
	int playerID;			/**< The Identification number of the new player */
	char playerName[21];	/**< The name of the new player */
};

/**
 * @struct sc_player_disconnect BustaNetMessages.h
 *
 * @brief (C) - Server notification that a player has disconnected
 *
 * The server sends out this message when a client has disconnected.  The message contains
 *	the identification number of the disconnected client as well as the reason for
 *	disconnection.
 */
struct sc_player_disconnect
{
	int playerID;			/**< The identification number of the disconnect client */
	char discMessage[101];	/**< The disconnection reason */
};

/**
 * @struct sc_player_lost BustaNetMessages.h
 *
 * @brief (E) - Server notification that a player has lost
 *
 * This message is sent out when a player loses in the current multiplayer game.  The
 *	message identifies the player that has lost.
 */
struct sc_player_lost
{
	int playerID;			/**< The identification number of the player who lost */
};

/**
 * @struct sc_sandwich_send BustaNetMessages.h
 *
 * @brief (H) - Server notification of a sent special sandwich
 *
 * The server sends this message out when it needs to notify a client that a special
 *	sandwich has been sent between players.  The message marks the player that sent
 *	the sandwich, the player to receive the sandwich, and the type of the sandwich
 *	sent.
 */
struct sc_sandwich_send
{
	int targetPlayer;			/**< The identification number of the player who receives the sandwich */
	int originatingPlayer;		/**< The identification number of the player who sent the sandwich */
	SpecialSandwich sandwich;	/**< The special sandwich sent */
};

/**
 * @struct sc_server_settings BustaNetMessages.h
 *
 * @brief (Contained in sc_join_response) - Server settings message
 *
 * This message contains information about the server and its settings.
 */
struct sc_server_settings
{
	char serverName[21];		/**< The server name */
	int maxPlayers;				/**< The number of players on the server */
};

/**
 * @struct sc_start_game BustaNetMessages.h
 *
 * @brief (B) - Server notification to start the game
 *
 * The server sends this message to a client to notify it of the beginning of the
 *	multiplayer game.
 */
struct sc_start_game
{

};

/**
 * @struct sc_join_response BustaNetMessages.h
 *
 * @brief (K) - Server response to a client's join attempt
 *
 * The server sends this message to a client that has attempted to join the multiplayer game.
 */
struct sc_join_response
{
	bool allowed;					/**< Whether the client is allowed to join the game */
	char reason[101];				/**< The reason to allow or disallow the client from joining */
	int clientID;					/**< The identification number assigned to the client if allowed */
	sc_server_settings settings;	/**< The settings of the server */
};

/**
 * @struct sc_player_ready BustaNetMessages.h
 *
 * @brief (L) - Notification that another player is ready to begin the game
 *
 * The server sends this message to a client to notify the client that a different player is ready
 *	to start the multiplayer game.
 */
struct sc_player_ready
{
	int playerID;			/**< Network identification number of the ready player */
};

/**
 * @struct sc_end_game BustaNetMessages.h
 *
 * @brief (M) - Notification that the game is over
 *
 * The game is over and all clients should transition to the high score/end game screen.  Followed by this type in an
 *  array of size max client of integers.  Then an array of the same size of int scores.
 */
struct sc_end_game
{
	
};

/**
 * @struct sc_ping BustaNetMessages.h
 *
 * @brief (Y) - Server requests a response from a client
 *
 * A server sends this message to confirm that the client is still responding.  This is followed
 *	by a 'pong' from the client signaling that the connection between the client and server
 *	is still available.
 */
struct sc_ping
{

};

/**
 * @struct sc_pong BustaNetMessages.h
 *
 * @brief (Z) - Server responds to a client ping request
 *
 * A server sends this message to a client in response to the client's 'ping' request.  This
 *	signals to the client that the server is still connected to it.
 */
struct sc_pong
{

};


#endif