/*	Filename:		BustaNetServer.cpp
 *	Description:	Busta' Sandwich server module implementation
 *
 */

#include <stdlib.h>
#include "BustaNetServer.h"

// ------------------------------- Public Functions ------------------------------------------
BustaNetServer::BustaNetServer()
{
	/* Initialize variables */
	clients = NULL;			// No clients
	clientSocketSet = NULL;	// No socket set for clients
	maxClients = 0;			// No clients allowed
	numClients = 0;			// No clients currently connected
	serverName = NULL;		// No server name
	serverSocketSet = NULL;	// No socket set for server messages
	servsock = NULL;		// No socket for server messages

	debug = false;		// DEBUG - debug output modifier
	extreme_debug = false;

	// PreAllocated Structs for network receives
	csJoinAttempt = new cs_join_attempt;
	csOpponentViewSelect = new cs_opponent_view_select;
	csSandwichSend = new cs_sandwich_send;
	csCharacterSelect = new cs_character_select;
	csCharacterCursor = new cs_character_cursor;
	
	scJoinResponse = new sc_join_response;
	scSandwichSend = new sc_sandwich_send;
	scPlayerReady = new sc_player_ready;
	scPlayerLost = new sc_player_lost;
	scPlayerDisconnect = new sc_player_disconnect;
	scNewConnection = new sc_new_connection;
	scCharacterSelectCursor = new sc_character_select_cursor;
	scCharacterSelected = new sc_character_selected;
}

BustaNetServer::~BustaNetServer()
{
	// Close tcp client sockets, and delete client list
	for (int i = 0; i < maxClients; i++) //maxClients should be deleted since they are created
	{	
		if(clients[i]->active)
		{
			SDLNet_TCP_DelSocket(clientSocketSet, clients[i]->socket);
			SDLNet_TCP_Close(clients[i]->socket);	// Close client socket
		}
		delete clients[i];							// Delete client entry
	}
	SDLNet_FreeSocketSet(clientSocketSet);
	clientSocketSet = NULL;

	// Close tcp server socket
	SDLNet_TCP_Close(servsock);
	SDLNet_FreeSocketSet(serverSocketSet);

	// Delete gamestates
	delete[] fullGameStates;
	delete[] briefGameStates;
	
	// Delete gameStats
	delete[] gameStatsRank;
	delete[] gameStatsScore;

	// Delete network structs 
	delete csJoinAttempt;
	delete scJoinResponse;
	delete csOpponentViewSelect;
	delete csSandwichSend;
	delete csCharacterSelect;
	delete csCharacterCursor;
	delete scSandwichSend;
	delete scPlayerReady;
	delete scPlayerLost;
	delete scPlayerDisconnect;
	delete scNewConnection;
	delete scCharacterSelectCursor;
	delete scCharacterSelected;

	// Shutdown SDL
	SDL_Quit();
}

char* BustaNetServer::init(Uint16 servport)
{
	client* oneClient;				// client record pointer

	// Create client list
	clients	= (client**)(malloc(maxClients*(sizeof(*oneClient))));
	for (int i = 0; i < maxClients; i++)
	{		
		// Create new client record
		oneClient = (struct client*)(malloc(sizeof(*oneClient)));

		// Set Client defaults
		oneClient->active = false;
		oneClient->ready = false;
		oneClient->char_selected = false;
		oneClient->still_playing = true;
		oneClient->last_checkin = 0;
		oneClient->character = C1;
		oneClient->focus_opp = NULL;
		oneClient->id_number = 0;
		oneClient->name[0] = NULL;
		oneClient->socket = NULL;

		// Add client record to client list
		clients[i] = oneClient;
	}

	// Initialize fullGameStates and briefGameStates
	fullGameStates =  new netGameState*[maxClients];
	briefGameStates = new netOpponent[maxClients];
	for(int i = 0; i < maxClients; i++)
	{
		//could be expanded more...
		fullGameStates[i] = new netGameState;
		fullGameStates[i]->score = 0;

		briefGameStates[i].clientID = -1;
		briefGameStates[i].health = 0;
	}
	
	// Initialize statistics
	gameStatsRank = new int[maxClients];
	gameStatsScore = new int[maxClients];
	for(int i = 0; i < maxClients; i++)
	{
		gameStatsRank[i] = -1;
		gameStatsScore[i] = -1;
	}


	// Initialize SDL
	if (SDL_Init(0) < 0)
		return SDL_GetError();		// Return error msg on SDL error

	// Initialize SDLnet
	if (SDLNet_Init() == -1) 
		return SDLNet_GetError();	// Return error msg on SDLNet error

	// Initialize SDL timer subsystem
	if (SDL_Init(SDL_INIT_TIMER) < 0)
		return SDL_GetError();		// Return error msg on SDL timer error

	// Allocate the socket set
	clientSocketSet = SDLNet_AllocSocketSet(maxClients);	// client socket set
	serverSocketSet = SDLNet_AllocSocketSet(1);				// server socket set

	// If we cannot create the server or client socket sets, return the error msg
	if ((clientSocketSet == NULL) || (serverSocketSet == NULL)) 
		return SDLNet_GetError();
		
	// Create the server socket
	SDLNet_ResolveHost(&serverIP, NULL, servport);			// resolve the server's ip with the specified port

	// Open the server socket
	servsock = SDLNet_TCP_Open(&serverIP);

	// If we cannot create the server socket, return the error msg
	if (servsock == NULL) 
		return SDLNet_GetError();

	// Add the server socket to the server socket set
	SDLNet_TCP_AddSocket(serverSocketSet, servsock);

	// return null on no error
	return NULL;
}

void BustaNetServer::sendAccept(int to_client, bool response, char* message)
{
	sendAccept(clients[to_client], response, message);
}

void BustaNetServer::sendCharacterSelectedAllO(client* sel_client, BustaModelCharacter character)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && clients[i]->id_number != sel_client->id_number)
			sendCharacterSelected(clients[i], sel_client, character);
	}
}

void BustaNetServer::sendCharacterSelectCursorAllO(client* sel_client, netPosition cursor)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && clients[i]->id_number != sel_client->id_number)
			sendCharacterSelectCursor(clients[i], sel_client, cursor);
	}
}

void BustaNetServer::sendCharacterSelectModeAll()
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active)
			sendCharacterSelectMode(clients[i]);
	}
}

void BustaNetServer::sendConnectedAllO(client* sel_client)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && (clients[i]->id_number != sel_client->id_number))
			sendConnected(clients[i], sel_client);
	}
}

void BustaNetServer::sendDisconnectedAllO(client* disc_client, char* message)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && (clients[i]->id_number != disc_client->id_number))
			sendDisconnected(clients[i], disc_client, message);
	}
}

void BustaNetServer::sendEndAll()
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active)
			sendEnd(clients[i]);
	}
}

void BustaNetServer::sendGameStartAll()
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active)
			sendGameStart(clients[i]);
	}
}

void BustaNetServer::sendPlayerLostAllO(client* lost_player)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && clients[i]->id_number != lost_player->id_number)
			sendPlayerLost(clients[i], lost_player);
	}
}

void BustaNetServer::sendPlayerReadyAllO(client* ready_player)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && clients[i]->id_number != ready_player->id_number)
			sendPlayerReady(clients[i], ready_player);
	}
}

void BustaNetServer::sendSandwichAllO(client* target_client, client* from_client, SpecialSandwich sandwich)
{
	for(int i = 0; i < maxClients; i++)
	{
		if(clients[i]->active && (clients[i]->id_number != from_client->id_number))
			sendSandwich(clients[i], target_client, from_client, sandwich);
	}
}

void BustaNetServer::sendUpdateAll()
{
	for(int i = 0; i < numClients; i++)
	{
		if(clients[i]->active && clients[i]->focus_opp)
			sendUpdate(clients[i]);
	}
}

void BustaNetServer::setDebug(bool OnOff)
{
	debug = OnOff;
}

void BustaNetServer::setExtrememDebug(bool OnOff)
{
	extreme_debug = OnOff;
}

void BustaNetServer::setMaxClients(int maxC)
{
	maxClients = maxC;
}

int BustaNetServer::getMaxClients()
{
	return maxClients;
}

void BustaNetServer::setName(char* servname)
{
	serverName = servname;
}

char* BustaNetServer::getName()
{
	return serverName;
}

int BustaNetServer::setStateAutoAccept(int num_clients)
{
	TCPsocket newsock;		// Socket to assign to new clients

	// Begin Accepting Connections
	for (;;)
	{
		// Check for input on server socket
		SDLNet_CheckSockets(serverSocketSet, 0);

		// If input on socket, we have a connection
		if (SDLNet_SocketReady(servsock))
		{
			// Keep the socket used to connect to the server
			newsock = SDLNet_TCP_Accept(servsock);

			// DEBUG - Notify console of connection attempt
			if (debug) 
				printf("Incoming Connection\n");

			addClient(*SDLNet_TCP_GetPeerAddress(newsock), newsock, "Default Name");
		}

		for (int i = 0; i < numClients; i++)		// Set the clients as ready
		{
			clients[i]->ready = true;
			clients[i]->focus_opp = fullGameStates[0];
		}

		
		// If there are enough clients who are ready, exit accept state
		if (numClients >= num_clients)
		{
			return 1;
		}

		// Wait to poll socket again (80 ms)
		SDL_Delay(80);
	}

	// Error return
	return -1;
}


int BustaNetServer::setStateAccept()
{
	TCPsocket newsock;		// Socket to assign to new clients
	char type;				// incoming message types

	// Begin Accepting Connections
	for (;;)
	{
		// Check for input on server socket
		SDLNet_CheckSockets(serverSocketSet, 0);

		// If input on socket, we have a connection
		if (SDLNet_SocketReady(servsock))
		{
			// Keep the socket used to connect to the server
			newsock = SDLNet_TCP_Accept(servsock);

			// DEBUG - Notify console of connection attempt
			if (debug)
				printf("Incoming Connection\n");

			// Wait to poll socket again (100 ms), gives client time to send out join msg
			SDL_Delay(100);

			SDLNet_TCP_Recv(newsock, &type, 1);			// receive the type of a message (should be a join)

			if (type == 'i')							// if it's a join msg
			{
				SDLNet_TCP_Recv(newsock, csJoinAttempt, sizeof(*csJoinAttempt));		// receive the join msg
				
				// Checking the server has room
				if(numClients < maxClients)
				{
					bool newClient = true;
					IPaddress newIP = *SDLNet_TCP_GetPeerAddress(newsock);

					//Checking for a duplicate client
					for(int i = 0; i < maxClients; i++)
						if(clients[i]->active && clients[i]->address.host == newIP.host)
							newClient = false;

					if(newClient)
					{
						// Add the client
						int id = addClient(newIP, newsock, csJoinAttempt->clientName);

						sendAccept(clients[id], true, "Connection accepted.");
					}
					else
					{
						// Deny the client for already being on the server
						client denied_client;
						denied_client.socket = newsock;

						sendAccept(&denied_client, false, "You are already connected.");
					}
				}
				else
				{
					// Deny the client for the server being full
					client denied_client;
					denied_client.socket = newsock;

					sendAccept(&denied_client, false, "Server is full.");
				}
			}
			else
			{
				// An unexpected packet encountered
				return 0;
			}
		}


		// Check for input on client socket set
		if( SDLNet_CheckSockets(clientSocketSet, 0) > 0 )
		{
			// If there is activity, check all the sockets
			for(int i = 0; i < maxClients; i++)
			{
				if( clients[i]->active && SDLNet_SocketReady(clients[i]->socket) )
				{
					// Recieve the type packet
					char type;
					SDLNet_TCP_Recv(clients[i]->socket, &type, 1);

					switch( type )
					{
					case 'g':
					{
						// Set the client to ready
						clients[i]->ready = true;

						if( debug )
							printf("Set client %d to ready\n",i);

						sendPlayerReadyAllO(clients[i]);
					} break;
					case 'd':
					{
						// Recieve a disconnect, deactivate the client, send out notification
						disconnectClient(clients[i], 2);
					} break;
					case 'y':
					{
						char pong = 'Z';
						SDLNet_TCP_Send(clients[i]->socket, &pong, 1);
					} break;
					case 'z':
					{
						//possibly timeout here
					} break;
					default:
					{
						// An unexpected packet encountered
						if( debug )
							printf("Unexpected packet: %c\n", type);
						return 0;
					}
					}
				}
			}
		}

		// If there are at least two clients who are ready, exit accept state
		if (numClients >= 2)
		{
			int clientsRdy = 0;							// number of ready clients

			for (int i = 0; i < maxClients; i++)		// Count number of ready clients
			{
				if (clients[i]->ready)
					clientsRdy++;
			}

			if (clientsRdy == numClients)				// If all clients ready, exit accept state
			{
				sendCharacterSelectModeAll();
				return 1;
			}
		}

		// Wait to poll socket again (10 ms)
		SDL_Delay(10);
	}

	// Error return
	return -1;
}

int BustaNetServer::setStateCharSelect()
{
	for(;;)
	{
		// Check for input on server socket
		if( SDLNet_CheckSockets(clientSocketSet, 0) > 0 )
		{
			// If there is activity, check all the sockets
			for(int i = 0; i < maxClients; i++)
			{
				if( clients[i]->active && SDLNet_SocketReady(clients[i]->socket) )
				{
					// Recieve the type packet
					char type;
					SDLNet_TCP_Recv(clients[i]->socket, &type, 1);

					switch( type )
					{
					case 'c':
					{
						// Recieve the character cursor
						SDLNet_TCP_Recv(clients[i]->socket, csCharacterCursor, sizeof(cs_character_cursor));
						
						if( debug )
							printf("Recieved a character cursor form %d: %d, %d\n", clients[i]->id_number, csCharacterCursor->cursorLoc.x1, csCharacterCursor->cursorLoc.y1);
						
						sendCharacterSelectCursorAllO(clients[i], csCharacterCursor->cursorLoc);
					} break;
					case 'b':
					{
						// Recieve the character selection
						SDLNet_TCP_Recv(clients[i]->socket, csCharacterSelect, sizeof(cs_character_select));

						clients[i]->char_selected = true;
						clients[i]->character = csCharacterSelect->character;

						if( debug )
							printf("Player %d selected a character\n", clients[i]->id_number);
						
						sendCharacterSelectedAllO(clients[i], csCharacterSelect->character);	
					} break;
					case 'd':
					{
						// Recieve a disconnect, deactivate the client, send out notification
						disconnectClient(clients[i], 2);
					} break;
					case 'y':
					{
						char pong = 'Z';
						SDLNet_TCP_Send(clients[i]->socket, &pong, 1);
					} break;
					case 'z':
					{
						//possibly timeout here
					} break;
					default:
					{
						// An unexpected packet encountered
						if( debug )
							printf("Unexpected packet: %c\n", type);
						return 0;
					}
					}
				}
			}
		}

		// check that there are still enough clients
		if(numClients < 2)
		{
			for(int i = 0; i < maxClients; i++)
			{
				if( clients[i]->active )
					disconnectClient(clients[i], 3);
			}
			SDL_Delay( 1000 );
			return 2;
		}

		// Check that all clients have selected
		int all_selected = 0;

		for(int i = 0; i < maxClients; i++)
		{
			if( clients[i]->active &&  clients[i]->char_selected )
				all_selected++;
		}
		
		if( all_selected == numClients )
		{
			// Send out the game start command
			sendGameStartAll();
			return 1;
		}

		// Wait to poll socket again (10 ms)
		SDL_Delay(10);
	}

	// Error return
	return -1;
}

int BustaNetServer::setStateGame()
{
	DWORD lastUpdateSent = GetTickCount();
	DWORD tmpClockTime;

	for( int i = 0; i < maxClients; i++ )
	{
		if(clients[i]->active)
			clients[i]->last_checkin = lastUpdateSent;
	}

	for(;;)
	{
		// Check for input on server socket
		if( SDLNet_CheckSockets(clientSocketSet, 0) > 0 )
		{
			// If there is activity, check all the sockets
			for(int i = 0; i < maxClients; i++)
			{
				SDLNet_CheckSockets(clientSocketSet, 0);
				if( clients[i]->active && SDLNet_SocketReady(clients[i]->socket) )
				{
					// Recieve the type packet
					char type;
					SDLNet_TCP_Recv(clients[i]->socket, &type, 1);

					switch( type )
					{
					case 'a':
					{
						// Recieve a netGameState update
						SDLNet_TCP_Recv(clients[i]->socket, fullGameStates[i], sizeof(netGameState));

						if( debug && extreme_debug )
							printf("Recieved game state from %d\n", clients[i]->id_number);

						clients[i]->last_checkin = GetTickCount();
					
						updateBriefGameState(i);
					} break;
					case 'd':
					{
						// Recieve a disconnect, deactivate the client, send out notification
						setClientLost(clients[i]);
						disconnectClient(clients[i], 2);
					} break;
					case 'e':
					{
						// Recieve a player lost message
						setClientLost(clients[i]);

						// Notify others client has lost
						sendPlayerLostAllO(clients[i]);
					} break;
					case 'f':
					{
						// Recieve an opponent view change and set it
						SDLNet_TCP_Recv(clients[i]->socket, csOpponentViewSelect, sizeof(cs_opponent_view_select));

						clients[i]->focus_opp = fullGameStates[csOpponentViewSelect->opponentID];
						
						if( debug )
							printf("Player %d set focus opponent of %d\n", i, csOpponentViewSelect->opponentID);
					} break;
					case 'h':
					{
						// Recieve a special sandwich and forward it along, notify the rest
						SDLNet_TCP_Recv(clients[i]->socket, csSandwichSend, sizeof(cs_sandwich_send));

						sendSandwichAllO(clients[csSandwichSend->targetPlayer], clients[i], csSandwichSend->sandwich);

						if (debug)
							printf("Special Sandwich Sent from %i to %i of type %i\n", i, csSandwichSend->targetPlayer, (int)csSandwichSend->sandwich);
					} break;
					default:
					{
						// An unexpected packet encountered
						if( debug )
							printf("Unexpected packet: %c\n", type);
						return 0;
					}
					}
				}
			}
		}


		// Send out an update every 500 ms
		if( GetTickCount() - lastUpdateSent >= 500 )
		{
			sendUpdateAll(); 
			lastUpdateSent = GetTickCount();
		}
		else
		{
			// Wait to poll socket again (1 ms)
			SDL_Delay(1);
		}


		// Check that clients are still playing
		int still_playing = 0;

		for(int i = 0; i < maxClients; i++)
		{
			if( clients[i]->active && clients[i]->still_playing )
				still_playing++;

			// Check for client timeout (5 s)
			if( clients[i]->active && (GetTickCount() - clients[i]->last_checkin) >= 5000 )
			{
				setClientLost(clients[i]);
				disconnectClient(clients[i], 1);
			}
		}
		
		if( still_playing == 1 )
		{
			for(int i = 0; i < maxClients; i++)
				if(clients[i]->active && clients[i]->still_playing)
					setClientLost(clients[i]);
			sendEndAll();
			SDL_Delay(5000);
			return 1;
		}
	}

	// Error return
	return -1;
}

// ------------------------------- Private Functions ------------------------------------------

int BustaNetServer::addClient(IPaddress IP, TCPsocket socket, char* clientName)
{
	int clientRecord;		// Open client record index in client list

	// Locate an empty client record
	for (clientRecord = 0; clientRecord < maxClients; clientRecord++)
	{
		if (!(clients[clientRecord]->active))
			break;								// break out once an open slot is found
	}

	// Create a record of the client
	clients[clientRecord]->active = true;
	clients[clientRecord]->address = IP;
	clients[clientRecord]->char_selected = false;
	clients[clientRecord]->focus_opp = NULL;
	clients[clientRecord]->id_number = clientRecord;
	strcpy(clients[clientRecord]->name, clientName);
	clients[clientRecord]->ready = false;
	clients[clientRecord]->socket = socket;

	// Add the client socket to the socket set
	SDLNet_TCP_AddSocket(clientSocketSet, clients[clientRecord]->socket);

	// Increment number of clients connected
	numClients++;

	// DEBUG - output connection info for client
	if (debug)
	{
		printf("Client Joined The Game\n");
		printf("\tIP Address: %d.%d.%d.%d\n",
			clients[clientRecord]->address.host&0xff,
			(clients[clientRecord]->address.host>>8)&0xff,
			(clients[clientRecord]->address.host>>16)&0xff,
			clients[clientRecord]->address.host>>24);
		printf("\tID Number: %i\n", clients[clientRecord]->id_number);
		printf("\tName: %s\n", clients[clientRecord]->name);
		printf("\n");
	}
	else
	{
		printf("Client %d (%s) Joined The Game\n", clients[clientRecord]->id_number, clients[clientRecord]->name);
	}

	return clientRecord;
}

void BustaNetServer::disconnectClient(client* sel_client, int reason)
{
	// deactivate the client, send out notification
	char buffer[100];
	
	switch( reason )
	{
	case 1: 
		{	// Notify the client they have been timed out, just in case
			sprintf(buffer, "Client %d (%s) timed out.", sel_client->id_number, sel_client->name); 
			sendDisconnected(sel_client, sel_client, buffer);
		} break;
	case 2: { sprintf(buffer, "Client %d (%s) disconnected.", sel_client->id_number, sel_client->name); } break;
	case 3: 
		{	// Notify the client the server is disconnecting them
			sprintf(buffer, "Client %d (%s) disconnected by server.", sel_client->id_number, sel_client->name);
			sendDisconnected(sel_client, sel_client, buffer);
		} break;
	}

	sel_client->active = false;
	sel_client->ready = false;
	SDLNet_TCP_DelSocket(clientSocketSet, sel_client->socket);
	SDLNet_TCP_Close(sel_client->socket);
	numClients--;

	printf("%s\n", buffer);
	sendDisconnectedAllO(sel_client, buffer);
}

void BustaNetServer::sendAccept(client* to_client, bool response, char* message)
{
	// send join response struct to you
	char type = 'K';
	sc_join_response msg;
	sc_server_settings settings;

	settings.maxPlayers = getMaxClients();
	strcpy(settings.serverName, getName());
	
	if(response)
		msg.clientID = to_client->id_number;
	else 
		msg.clientID = -1;
	msg.allowed = response;
	strcpy(msg.reason, message);
	msg.settings = settings;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, &msg, sizeof(sc_join_response));
	
	if( debug )
		printf("Sent sc_join_response to %d\n", to_client->id_number );

	// send a notification to everyone else if you were accepted
	if(response)
	{
		sendConnectedAllO(to_client);

		// notify the new client of the other players on the server
		for(int i = 0; i < maxClients; i++)
		{
			if(clients[i]->active && (to_client->id_number != clients[i]->id_number))
			{
				sendConnected(to_client, clients[i]);
				if(clients[i]->ready)
					sendPlayerReady(to_client, clients[i]);
			}
		}
	}
}

void BustaNetServer::sendCharacterSelected(client* to_client, client* sel_client, BustaModelCharacter character)
{
	char type = 'F';

	scCharacterSelected->selectingPlayer = sel_client->id_number;
	scCharacterSelected->character = character;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scCharacterSelected, sizeof(sc_character_selected));

	if( debug )
		printf("Sent sc_character_selected to %d\n", to_client->id_number );
}

void BustaNetServer::sendCharacterSelectMode(client* to_client)
{
	char type = 'D';

	SDLNet_TCP_Send(to_client->socket, &type, 1);

	if( debug )
		printf("Sent Character Select Mode to %d\n", to_client->id_number );
}

void BustaNetServer::sendCharacterSelectCursor(client* to_client, client* sel_client, netPosition cursor)
{
	char type = 'G';

	scCharacterSelectCursor->playerID= sel_client->id_number;
	scCharacterSelectCursor->cursorLoc.x1 = cursor.x1;
	scCharacterSelectCursor->cursorLoc.y1 = cursor.y1;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scCharacterSelectCursor, sizeof(sc_character_select_cursor));

	if( debug )
		printf("Sent sc_character_select_cursor to %d\n", to_client->id_number );
}

void BustaNetServer::sendConnected(client* to_client, client* sel_client)
{
	char type = 'I';

	scNewConnection->playerID = sel_client->id_number;
	strcpy(scNewConnection->playerName, sel_client->name);

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scNewConnection, sizeof(sc_new_connection));

	if( debug )
		printf("Sent sc_new_connection of %d to %d\n", sel_client->id_number, to_client->id_number );
}

void BustaNetServer::sendDisconnected(client* to_client, client* disc_client, char* message)
{
	char type = 'C';

	scPlayerDisconnect->playerID = disc_client->id_number;
	strcpy(scPlayerDisconnect->discMessage, message);

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scPlayerDisconnect, sizeof(sc_player_disconnect));

	if( debug )
		printf("Sent sc_player_disconnect to %d\n", to_client->id_number );
}

void BustaNetServer::sendEnd(client* to_client)
{
	char type = 'M';

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, gameStatsRank, maxClients*sizeof(int));
	SDLNet_TCP_Send(to_client->socket, gameStatsScore, maxClients*sizeof(int));

	if( debug ) {
		printf("Sent end game to %d\n", to_client->id_number );
		printf("Rank: ");
		for(int i = 0; i < maxClients; i++)
			printf("%d ", gameStatsRank[i]);
		printf("\nScore: ");
		for(int i = 0; i < maxClients; i++)
			printf("%d ", gameStatsScore[i]);
		printf("\n");
	}
}

void BustaNetServer::sendGameStart(client* to_client)
{
	char type = 'B';

	SDLNet_TCP_Send(to_client->socket, &type, 1);

	if( debug )
		printf("Sent game start to %d\n", to_client->id_number );
}

void BustaNetServer::sendPlayerLost(client* to_client, client* lost_player)
{
	char type = 'E';

	scPlayerLost->playerID = lost_player->id_number;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scPlayerLost, sizeof(sc_player_lost));

	if( debug )
		printf("Sent sc_player_lost to %d\n", to_client->id_number );
}

void BustaNetServer::sendPlayerReady(client* to_client, client* ready_client)
{
	char type = 'L';

	scPlayerReady->playerID = ready_client->id_number;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scPlayerReady, sizeof(sc_player_ready));

	if( debug )
		printf("Sent sc_player_ready to %d\n", to_client->id_number );
}

void BustaNetServer::sendSandwich(client* to_client, client* target_client, client* from_client, SpecialSandwich sandwich)
{
	char type = 'H';

	scSandwichSend->originatingPlayer = from_client->id_number;
	scSandwichSend->targetPlayer = target_client->id_number;
	scSandwichSend->sandwich = sandwich;

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, scSandwichSend, sizeof(sc_sandwich_send));

	if( debug )
		printf("Sent sc_sandwich_send to %d\n", to_client->id_number );
}

void BustaNetServer::sendUpdate(client* to_client)
{
	char type = 'A';

	SDLNet_TCP_Send(to_client->socket, &type, 1);
	SDLNet_TCP_Send(to_client->socket, to_client->focus_opp, sizeof(netGameState));
	SDLNet_TCP_Send(to_client->socket, briefGameStates, maxClients*sizeof(netOpponent));

	if( debug && extreme_debug )
		printf("Sent update to %d\n", to_client->id_number );
}

void BustaNetServer::setClientLost(client* lost_player)
{
	// Flag the losing player and update the stats
	lost_player->still_playing = false;
	
	int i;
	for(i = 0; i < maxClients; i++ )
		if(gameStatsRank[i] == -1)
			break;

	gameStatsRank[i] = lost_player->id_number;
	gameStatsScore[i] = fullGameStates[lost_player->id_number]->score;

	if( debug )
		printf("Player %d lost.\n", lost_player->id_number);
}

void BustaNetServer::updateBriefGameState(int sel_client)
{
	briefGameStates[sel_client].clientID = clients[sel_client]->id_number;
	briefGameStates[sel_client].health = fullGameStates[sel_client]->chef.health;
	briefGameStates[sel_client].tips.tips = fullGameStates[sel_client]->tips.tips;
	briefGameStates[sel_client].tips.fullamount = fullGameStates[sel_client]->tips.fullamount;

	if( debug && extreme_debug )
		printf("Updated brief gamestate of %d\n", sel_client );
}