/*	Filename:		BustaNetServer.h
 *	Description:	Busta' Sandwich server module
 *
 */

#ifndef BUSTA_SANDWICH_SERVER
#define BUSTA_SANDWICH_SERVER

#include "../SDL/include/SDL.h"
#include "../SDL/include/SDL_net.h"
#include <string.h>
#include <Windows.h>
#include "BustaNetMessages.h"

/**
 * @struct client BustaNetServer.h
 *
 * @brief Representation of a client connected to the server
 *
 * The struct contains information regarding a client connected to the server.  This is used by the
 *	server to maintain an accurate listing of all connected clients.
 */
struct client
{
	bool active;					/**< Whether this client record is in use by a real client */
	bool ready;						/**< Whether a ready msg has been received from this client */
	bool char_selected;				/**< Whether the client has selected a character */
	bool still_playing;				/**< Whether the client is still an active player of has lost */
	DWORD last_checkin;				/**< The last time the client checked in with an update > */
	BustaModelCharacter character;	/**< The character icon the player has choosen */
	netGameState* focus_opp;		/**< The oppononent that the player is viewing in full */
	IPaddress address;				/**< The ip address and port of the client */
	TCPsocket socket;				/**< The socket used to connect to this client */
	int id_number;					/**< The identification number of the client */
	char name[21];						/**< The client's player's name */
};

/**
 * @class BustaNetServer
 * @brief Server network module for Busta'Sandwich
 *
 * Standalone server class for hosting and handling multiplayer games of Busta'Sandwich..
 */
class BustaNetServer
{
	public:
		/**
		 * @brief Constructor
		 *
		 */
		BustaNetServer();

		/**
		 * @brief Destructor
		 *
		 */
		~BustaNetServer();

		/**
		 * @brief Initialize the server
		 *
		 * Initializes SDL and SDLNet.  A socket to connect to the server is created for the server's
		 *	ip address on the specified port.
		 *
		 * @param servport The port number to host the game on
		 *
		 * @return Returns a error message if an error occurs during initialization.  On no error
		 *	null is returned.
		 */
		char* init(Uint16 servport);

		/**
		 * @brief Sends a connection response to a connecting client
		 *
		 * Responds to a client's connection attempt.  The client is sent a message detailing whether
		 *	or not the connection was allowed, a message describing why or why not the connection was
		 *	allowed, and the player's identification number if the connection was accepted.  
		 *
		 * @param to_client The id number of the accepted client
		 * @param response Response to the connection attempt.  True for allow, false for deny
		 * @param message Description of the server's response; why allowed or denied
		 */
		void sendAccept(int to_client, bool response, char* message);

		/**
		 * @brief Sends a character selection notification to all connected clients
		 *
		 * Sends a notification message to all clients except the selected client that
		 *	the player has selected a character and what it is.
		 *
		 * @param sel_client The pointer to the selected client's struct
		 * @param character The character selected by the client
		 */
		void sendCharacterSelectedAllO(client* sel_client, BustaModelCharacter character);

		/**
		 * @brief Sends a character selection cursor udpate to all connected clients
		 *
		 * Sends an update message to all clients except the selected client that
		 *	the player has moved the cursor and the location.
		 *
		 * @param sel_client The pointer to the selected client's struct
		 * @param cursor The character cursor location of the client
		 */
		void sendCharacterSelectCursorAllO(client* sel_client, netPosition cursor);

		/**
		 * @brief Notifies all connected clients that multiplayer character selection is taking place
		 *
		 * A message is sent to all clients connected to the server that all clients are to now select
		 *	a multiplayer character.
		 */
		void sendCharacterSelectModeAll();
		
		/**
		 * @brief Notifies all connected clients another player has joined the game
		 *
		 * A message that notifies all of the other clients that a player has connected.  Used
		 *  in the connection screen for example.
		 */
		void sendConnectedAllO(client* sel_client);

		/**
		 * @brief Sends a disconnect notification to all connected clients
		 *
		 * Sends a notification message to all clients that are still connected when a client disconnects
		 *	from the server.  Includes details as to why client disconnected.
		 *
		 * @param disc_client The pointer to the disconnected client's struct
		 * @param message Reason for client disconnection
		 */
		void sendDisconnectedAllO(client* disc_client, char* message);

		/**
		 * @brief Sends an end game message to all players
		 *
		 * Needs some commenting...
		 */
		void sendEndAll();

		/**
		 * @brief Sends a notification to all client when one client has lost in the current game
		 *
		 * A notification that a client has lost in the current multiplayer game is sent to all other
		 *	connected clients.  The player who has lost is not notified.
		 *
		 * @param lost_player The pointer to the player's struct who has lost
		 */
		void sendPlayerLostAllO(client* lost_player);

		/**
		 * @brief Sends a notification to all client when a player has selected ready
		 *
		 * A notification that a client has selected ready in the connection screen
		 *
		 * @param ready_player The pointer to the player's struct who is ready
		 */
		void sendPlayerReadyAllO(client* ready_player);

		/**
		 * @brief Sends a notification to all client opponents that a special sandwich has been sent to a player
		 *
		 * A notification that a client has sent a special sandwich to another opponent
		 *
		 * @param target_client The pointer to the client's struct who recieved the sandwich
		 * @param from_client The pointer to the client's struct who sent the sandwich
		 * @param sandwich The type of special sandwich sent
		 */
		void sendSandwichAllO(client* target_client, client* from_client, SpecialSandwich sandwich);

		/**
		 * @brief Notifies all connected clients to start the multiplayer game
		 *
		 * A message is sent to all connected clients that the multiplayer game is now starting.
		 */
		void sendGameStartAll();

		/**
		 * @brief Sends client game state information to all connected clients
		 *
		 * Sends game state information to all connected clients.  This includes the brief information
		 *	required by each player as well as the detailed game state requested by each client.
		 */
		void sendUpdateAll();

		/**
		 * @brief Sets the server in debug mode
		 *
		 * Turns on debugging output to the console for the server.
		 *
		 * @param OnOff Boolean to turn on or off debugging output
		 */
		void setDebug(bool OnOff);

		/**
		 * @brief Sets the server in extreme debug mode
		 *
		 * Turns on extreme debugging output to the console for the server.
		 *
		 * @param OnOff Boolean to turn on or off extreme debugging output
		 */
		void setExtrememDebug(bool OnOff);

		/**
		 * @brief Sets the maximum number of allowed connections
		 *
		 * Specifies the maximum number of simultaneous connections allowed by clients at any given time.
		 *
		 * @param maxC The maximum number of clients to allow
		 */
		void setMaxClients(int maxC);
		
		/**
		 * @brief Gets the maximum number of allowed connections
		 *
		 * Returns the maximum number of simultaneous connections allowed by clients at any given time.
		 */
		int getMaxClients();

		/**
		 * @brief Sets the name of the server
		 *
		 * Assigns the server an identification name.
		 *
		 * @param servname The name to give the server
		 */
		void setName(char* servname);

		/**
		 * @brief Gets the name of the server
		 *
		 * Returns the server identification name.
		 */
		char* getName();

		/**
		 * @brief Auto accepts clients and adds them to the socket set
		 *
		 * The server automatically accepts clients and adds them to the socket set.  Used only for testing
		 *  other functionality.  Does not send any of the notification messages of a standard accept loop.
		 *
		 * @param num_clients The number of clients to wait for
		 *
		 * @return Returns an integer greater than or equal to zero if the above condition is met.  A
		 *	negative integer is returned on error in this mode.
		 */
		int setStateAutoAccept(int num_clients);

		/**
		 * @brief Sets the server in client connection accepting mode
		 *
		 * The server is now in a state to accept incoming TCP connections from clients.  The server may
		 *	leave this mode when all connected clients have sent a ready notification to the server and
		 *	there is at least two connected clients.
		 *
		 * @return Returns an integer greater than or equal to zero if the above condition is met.  A
		 *	negative integer is returned on error in this mode.
		 */
		int setStateAccept();

		/**
		 * @brief Sets the server in client character selection mode
		 *
		 * The server is now in a state that allows clients to select a multiplayer character.  The server
		 *	leaves this mode when all connected clients have sent a selected character to the server.  The server
		 *	may also leave this mode if client disconnects occur leaving less than two connected clients.
		 *
		 * @return Returns an integer greater than zero if all connected clients have selected a character.  Returns
		 *	zero if client disconnects leaves less than two players.  Returns a negative value on error.
		 */
		int setStateCharSelect();

		/**
		 * @brief Sets the server in game mode
		 *
		 * The server is now in multiplayer game mode.  All clients are currently playing the game.  The server
		 *	leaves this mode when all but one player has lost in the game or less than two clients remain connected
		 *	to the server while in this mode.
		 *
		 * @return Returns an integer greater than zero if a winner is determined.  Returns 0 if less than two clients
		 *	are connected and a negative value on error.
		 */
		int setStateGame();

	private:
		IPaddress serverIP;					// Server's IP address and port
		char* serverName;					// Server's name

		int maxClients;						// Maximum number of allow clients to the server
		int numClients;						// Number of clients currently connected to server
		client** clients;					// Array of all connected clients		

		SDLNet_SocketSet clientSocketSet;	// Socketset for client connections
		SDLNet_SocketSet serverSocketSet;	// Socketset for server socket
		TCPsocket servsock;					// Server socket for incoming connections

		netGameState** fullGameStates;		// Stores the full game states for each player
		netOpponent* briefGameStates;		// Stores the brief game state for each player

		int* gameStatsRank;					// Stores the order in which the clients dropped out of the game
		int* gameStatsScore;				// Stores the scores of clients in the order the dropped out

		int  addClient(IPaddress IP, TCPsocket socket, char* clientName);
		void disconnectClient(client* sel_client, int reason);
		void sendAccept(client* to_client, bool response, char* message);
		void sendCharacterSelected(client* to_client, client* sel_client, BustaModelCharacter character);
		void sendCharacterSelectCursor(client* to_client, client* sel_client, netPosition cursor);
		void sendCharacterSelectMode(client* to_client);
		void sendConnected(client* to_client, client* sel_client);
		void sendDisconnected(client* to_client, client* disc_client, char* message);
		void sendEnd(client* to_client);
		void sendPlayerLost(client* to_client, client* lost_player);
		void sendPlayerReady(client* to_client, client* ready_client);
		void sendSandwich(client* to_client, client* target_client, client* from_client, SpecialSandwich sandwich);
		void sendGameStart(client* to_client);
		void sendUpdate(client* to_client);
		void setClientLost(client* lost_player);
		void updateBriefGameState(int sel_client);

		bool debug;		// DEBUG NOTIFIER
		bool extreme_debug;

		/** PreAllocated structs for network receiving **/
		cs_join_attempt* csJoinAttempt;
		cs_opponent_view_select* csOpponentViewSelect;
		cs_sandwich_send* csSandwichSend;
		cs_character_select* csCharacterSelect;
		cs_character_cursor* csCharacterCursor;

		sc_join_response* scJoinResponse;
		sc_sandwich_send* scSandwichSend;
		sc_player_ready* scPlayerReady;
		sc_player_lost* scPlayerLost;
		sc_player_disconnect* scPlayerDisconnect;
		sc_new_connection* scNewConnection;
		sc_character_select_cursor* scCharacterSelectCursor;
		sc_character_selected* scCharacterSelected;
};

#endif
